@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -96,8 +91,9 @@ class CControllerPopupTabFilterUpdate extends CController {
 		];
 
 		if ($this->getInput('support_custom_time', 0) && $properties['filter_custom_time']) {
-			$properties['from'] = $this->getInput('tabfilter_from', '');
-			$properties['to'] = $this->getInput('tabfilter_to', '');
+			$properties['from'] = $this->getInput('tabfilter_from');
+			$properties['to'] = $this->getInput('tabfilter_to');
+			$properties['filter_custom_time_label'] = relativeDateToText($properties['from'], $properties['to']);
 		}
 
 		$filter = (new CTabFilterProfile($idx, []))->read();
