@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 ?>
 
@@ -38,10 +33,6 @@ window.proxy_edit_popup = new class {
 			this.form.querySelector('#tls_connect input:checked').value == <?= HOST_ENCRYPTION_PSK ?>
 				|| document.getElementById('tls_accept_psk').checked;
 
-		document
-			.getElementById('useip')
-			.addEventListener('change', () => this._updateInterface());
-
 		if (this.display_change_psk) {
 			document
 				.getElementById('tls-psk-change')
@@ -52,7 +43,10 @@ window.proxy_edit_popup = new class {
 			}
 		}
 
-		for (const id of ['status', 'tls_connect', 'tls_accept_psk', 'tls_accept_certificate']) {
+		jQuery('#proxy_groupid').on('change', () => this._update());
+
+		for (const id of ['operating_mode', 'tls_connect', 'tls_accept_psk', 'tls_accept_certificate',
+				'custom_timeouts']) {
 			document
 				.getElementById(id)
 				.addEventListener('change', () => this._update());
@@ -61,18 +55,7 @@ window.proxy_edit_popup = new class {
 		this._update();
 
 		document.getElementById('proxy-form').style.display = '';
-		document.getElementById('host').focus();
-	}
-
-	_updateInterface() {
-		if (this.form.querySelector('#useip input:checked').value == <?= INTERFACE_USE_IP ?>) {
-			document.querySelector('.js-interface input[name="ip"]').setAttribute('aria-required', 'true');
-			document.querySelector('.js-interface input[name="dns"]').removeAttribute('aria-required');
-		}
-		else {
-			document.querySelector('.js-interface input[name="ip"]').removeAttribute('aria-required');
-			document.querySelector('.js-interface input[name="dns"]').setAttribute('aria-required', 'true');
-		}
+		document.getElementById('name').focus();
 	}
 
 	_changePsk() {
@@ -92,22 +75,29 @@ window.proxy_edit_popup = new class {
 	}
 
 	_update() {
-		const status_active = document.querySelector('#status input:checked').value == <?= HOST_STATUS_PROXY_ACTIVE ?>;
+		const $proxy_group = jQuery('#proxy_groupid').multiSelect('getData');
+
+		for (const element of this.form.querySelectorAll('.js-local-address')) {
+			element.style.display = $proxy_group.length ? '' : 'none';
+		}
+
+		const operating_mode_active =
+			document.querySelector('#operating_mode input:checked').value == <?= PROXY_OPERATING_MODE_ACTIVE ?>;
 
 		for (const element of this.form.querySelectorAll('.js-interface')) {
-			element.style.display = status_active ? 'none' : '';
+			element.style.display = operating_mode_active ? 'none' : '';
 		}
 
 		for (const element of this.form.querySelectorAll('.js-proxy-address')) {
-			element.style.display = status_active ? '' : 'none';
+			element.style.display = operating_mode_active ? '' : 'none';
 		}
 
 		for (const element of this.form.querySelectorAll('#tls_connect input')) {
-			element.disabled = status_active;
+			element.disabled = operating_mode_active;
 		}
 
 		for (const id of ['tls_accept_none', 'tls_accept_psk', 'tls_accept_certificate']) {
-			document.getElementById(id).disabled = !status_active;
+			document.getElementById(id).disabled = !operating_mode_active;
 		}
 
 		const tls_connect = this.form.querySelector('#tls_connect input:checked').value;
@@ -123,7 +113,7 @@ window.proxy_edit_popup = new class {
 
 		for (const id of ['tls_issuer', 'tls_subject']) {
 			document.getElementById(id).disabled =
-				!(status_active && tls_accept_certificate || !status_active && tls_connect_certificate);
+				!(operating_mode_active && tls_accept_certificate || !operating_mode_active && tls_connect_certificate);
 		}
 
 		if (this.display_change_psk) {
@@ -136,7 +126,7 @@ window.proxy_edit_popup = new class {
 			}
 
 			document.getElementById('tls-psk-change').disabled =
-				!(status_active && tls_accept_psk || !status_active && tls_connect_psk);
+				!(operating_mode_active && tls_accept_psk || !operating_mode_active && tls_connect_psk);
 		}
 		else {
 			for (const element of this.form.querySelectorAll('.js-tls-psk-identity, .js-tls-psk')) {
@@ -145,15 +135,24 @@ window.proxy_edit_popup = new class {
 
 			for (const id of ['tls_psk_identity', 'tls_psk']) {
 				document.getElementById(id).disabled =
-					!(status_active && tls_accept_psk || !status_active && tls_connect_psk);
+					!(operating_mode_active && tls_accept_psk || !operating_mode_active && tls_connect_psk);
 			}
 		}
+
+		const custom_timeouts_enabled =
+			this.form.querySelector('#custom_timeouts input:checked').value == <?= ZBX_PROXY_CUSTOM_TIMEOUTS_ENABLED ?>;
+
+		for (const id of ['timeout_zabbix_agent', 'timeout_simple_check', 'timeout_snmp_agent',
+				'timeout_external_check', 'timeout_db_monitor', 'timeout_http_agent', 'timeout_ssh_agent',
+				'timeout_telnet_agent', 'timeout_script', 'timeout_browser']) {
+			document.getElementById(id).readOnly = !custom_timeouts_enabled;
+		}
 	}
 
 	refreshConfig() {
 		const curl = new Curl('zabbix.php');
 		curl.setArgument('action', 'proxy.config.refresh');
-		curl.setArgument('<?= CCsrfTokenHelper::CSRF_TOKEN_NAME ?>', <?= json_encode(CCsrfTokenHelper::get('proxy')) ?>);
+		curl.setArgument(CSRF_TOKEN_NAME, <?= json_encode(CCsrfTokenHelper::get('proxy')) ?>);
 
 		this._post(curl.getUrl(), {proxyids: [this.proxyid]}, (response) => {
 			for (const element of this.form.parentNode.children) {
@@ -174,17 +173,19 @@ window.proxy_edit_popup = new class {
 
 		this.overlay.unsetLoading();
 		this.overlay.setProperties({title, buttons});
+		this.overlay.recoverFocus();
+		this.overlay.containFocus();
 	}
 
 	delete() {
 		const curl = new Curl('zabbix.php');
 		curl.setArgument('action', 'proxy.delete');
-		curl.setArgument('<?= CCsrfTokenHelper::CSRF_TOKEN_NAME ?>', <?= json_encode(CCsrfTokenHelper::get('proxy')) ?>);
+		curl.setArgument(CSRF_TOKEN_NAME, <?= json_encode(CCsrfTokenHelper::get('proxy')) ?>);
 
 		this._post(curl.getUrl(), {proxyids: [this.proxyid]}, (response) => {
 			overlayDialogueDestroy(this.overlay.dialogueid);
 
-			this.dialogue.dispatchEvent(new CustomEvent('dialogue.delete', {detail: response.success}));
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: response}));
 		});
 	}
 
@@ -203,8 +204,11 @@ window.proxy_edit_popup = new class {
 			fields.clone_psk = false;
 		}
 
-		for (const name of ['host', 'ip', 'dns', 'port', 'proxy_address', 'description', 'tls_psk_identity', 'tls_psk',
-				'tls_issuer', 'tls_subject']) {
+		for (const name of ['name', 'local_address', 'local_port', 'allowed_addresses', 'address', 'port',
+				'description', 'tls_psk_identity', 'tls_psk', 'tls_issuer', 'tls_subject', 'timeout_zabbix_agent',
+				'timeout_simple_check', 'timeout_snmp_agent', 'timeout_external_check', 'timeout_db_monitor',
+				'timeout_http_agent', 'timeout_ssh_agent', 'timeout_telnet_agent', 'timeout_script',
+				'timeout_browser']) {
 			if (name in fields) {
 				fields[name] = fields[name].trim();
 			}
@@ -216,7 +220,7 @@ window.proxy_edit_popup = new class {
 		this._post(curl.getUrl(), fields, (response) => {
 			overlayDialogueDestroy(this.overlay.dialogueid);
 
-			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: response.success}));
+			this.dialogue.dispatchEvent(new CustomEvent('dialogue.submit', {detail: response}));
 		});
 	}
 
