@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -50,14 +45,30 @@ abstract class CItemGeneral extends CApiService {
 	 *
 	 * @var array
 	 */
-	protected const PREPROC_TYPES_WITH_PARAMS = [];
+	protected const PREPROC_TYPES_WITH_PARAMS = [
+		ZBX_PREPROC_MULTIPLIER, ZBX_PREPROC_RTRIM, ZBX_PREPROC_LTRIM, ZBX_PREPROC_TRIM, ZBX_PREPROC_REGSUB,
+		ZBX_PREPROC_XPATH, ZBX_PREPROC_JSONPATH, ZBX_PREPROC_VALIDATE_RANGE, ZBX_PREPROC_VALIDATE_REGEX,
+		ZBX_PREPROC_VALIDATE_NOT_REGEX, ZBX_PREPROC_ERROR_FIELD_JSON, ZBX_PREPROC_ERROR_FIELD_XML,
+		ZBX_PREPROC_ERROR_FIELD_REGEX, ZBX_PREPROC_THROTTLE_TIMED_VALUE, ZBX_PREPROC_SCRIPT,
+		ZBX_PREPROC_PROMETHEUS_PATTERN, ZBX_PREPROC_PROMETHEUS_TO_JSON, ZBX_PREPROC_CSV_TO_JSON,
+		ZBX_PREPROC_STR_REPLACE, ZBX_PREPROC_VALIDATE_NOT_SUPPORTED, ZBX_PREPROC_SNMP_WALK_VALUE,
+		ZBX_PREPROC_SNMP_WALK_TO_JSON, ZBX_PREPROC_SNMP_GET_VALUE
+	];
 
 	/**
 	 * A list of preprocessing types that supports the error handling.
 	 *
 	 * @var array
 	 */
-	protected const PREPROC_TYPES_WITH_ERR_HANDLING = [];
+	protected const PREPROC_TYPES_WITH_ERR_HANDLING = [
+		ZBX_PREPROC_MULTIPLIER, ZBX_PREPROC_REGSUB, ZBX_PREPROC_BOOL2DEC, ZBX_PREPROC_OCT2DEC, ZBX_PREPROC_HEX2DEC,
+		ZBX_PREPROC_DELTA_VALUE, ZBX_PREPROC_DELTA_SPEED, ZBX_PREPROC_XPATH, ZBX_PREPROC_JSONPATH,
+		ZBX_PREPROC_VALIDATE_RANGE, ZBX_PREPROC_VALIDATE_REGEX, ZBX_PREPROC_VALIDATE_NOT_REGEX,
+		ZBX_PREPROC_ERROR_FIELD_JSON, ZBX_PREPROC_ERROR_FIELD_XML, ZBX_PREPROC_ERROR_FIELD_REGEX,
+		ZBX_PREPROC_PROMETHEUS_PATTERN, ZBX_PREPROC_PROMETHEUS_TO_JSON, ZBX_PREPROC_CSV_TO_JSON,
+		ZBX_PREPROC_VALIDATE_NOT_SUPPORTED, ZBX_PREPROC_XML_TO_JSON, ZBX_PREPROC_SNMP_WALK_VALUE,
+		ZBX_PREPROC_SNMP_WALK_TO_JSON, ZBX_PREPROC_SNMP_GET_VALUE
+	];
 
 	/**
 	 * A list of supported item types.
@@ -74,11 +85,11 @@ abstract class CItemGeneral extends CApiService {
 	protected const VALUE_TYPE_FIELD_NAMES = [];
 
 	/**
-	 * Maximum number of inheritable items per iteration.
+	 * Maximum number of items per iteration.
 	 *
 	 * @var int
 	 */
-	protected const INHERIT_CHUNK_SIZE = 1000;
+	protected const CHUNK_SIZE = 1000;
 
 	/**
 	 * @abstract
@@ -168,7 +179,7 @@ abstract class CItemGeneral extends CApiService {
 				}
 
 				$params_types = [ITEM_TYPE_DB_MONITOR, ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_CALCULATED,
-					ITEM_TYPE_SCRIPT
+					ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER
 				];
 
 				if (in_array($item['type'], $params_types) && !in_array($db_item['type'], $params_types)) {
@@ -177,13 +188,14 @@ abstract class CItemGeneral extends CApiService {
 
 				$delay_types = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL, ITEM_TYPE_ZABBIX_ACTIVE,
 					ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_IPMI, ITEM_TYPE_SSH, ITEM_TYPE_TELNET,
-					ITEM_TYPE_CALCULATED, ITEM_TYPE_JMX, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_SNMP, ITEM_TYPE_SCRIPT
+					ITEM_TYPE_CALCULATED, ITEM_TYPE_JMX, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_SNMP, ITEM_TYPE_SCRIPT,
+					ITEM_TYPE_BROWSER
 				];
 
 				if (in_array($item['type'], $delay_types)) {
 					if (!in_array($db_item['type'], $delay_types)
 							|| ($db_item['type'] == ITEM_TYPE_ZABBIX_ACTIVE
-								&& strncmp($db_item['key_'], 'mqtt.get', 8) === 0)) {
+								&& strncmp($db_item['key_'], 'mqtt.get', 8) == 0)) {
 						$item += array_intersect_key($db_item, array_flip(['delay']));
 					}
 				}
@@ -214,7 +226,7 @@ abstract class CItemGeneral extends CApiService {
 					$item += array_intersect_key($db_item, array_flip(['jmx_endpoint']));
 				}
 
-				if ($item['type'] == ITEM_TYPE_SNMP && $db_item['type'] != ITEM_TYPE_SNMP) {
+				if ($item['type'] == ITEM_TYPE_SNMP) {
 					$item += array_intersect_key($db_item, array_flip(['snmp_oid']));
 				}
 
@@ -253,44 +265,83 @@ abstract class CItemGeneral extends CApiService {
 			}
 
 			if (array_key_exists('query_fields', $item)) {
-				foreach ($item['query_fields'] as $query_field) {
-					if (count($query_field) != 1 || key($query_field) === '') {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
-							'/'.($i + 1).'/query_fields', _('nonempty key and value pair expected'))
-						);
-					}
-				}
-
-				$item['query_fields'] = $item['query_fields'] ? json_encode($item['query_fields']) : '';
-
-				if (strlen($item['query_fields']) > DB::getFieldLength('items', 'query_fields')) {
+				if (strlen(self::prepareQueryFieldsForDb($item['query_fields']))
+						> DB::getFieldLength('items', 'query_fields')) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
 						'/'.($i + 1).'/query_fields', _('value is too long')
 					));
 				}
+
+				$sortorder = 0;
+				$fields = [];
+
+				foreach ($item['query_fields'] as $field) {
+					$fields[++$sortorder] = ['sortorder' => $sortorder] + $field;
+				}
+
+				$item['query_fields'] = $fields;
 			}
 
 			if (array_key_exists('headers', $item)) {
-				foreach ($item['headers'] as $name => $value) {
-					if (trim($name) === '' || !is_string($value) || $value === '') {
-						self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
-							'/'.($i + 1).'/headers', _('nonempty key and value pair expected')
-						));
-					}
-				}
-
-				$item['headers'] = self::headersArrayToString($item['headers']);
-
-				if (strlen($item['headers']) > DB::getFieldLength('items', 'headers')) {
+				if (strlen(self::prepareHeadersForDb($item['headers'])) > DB::getFieldLength('items', 'headers')) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
 						'/'.($i + 1).'/headers', _('value is too long')
 					));
 				}
+
+				$sortorder = 0;
+				$fields = [];
+
+				foreach ($item['headers'] as $field) {
+					$fields[++$sortorder] = ['sortorder' => $sortorder] + $field;
+				}
+
+				$item['headers'] = $fields;
 			}
 		}
 		unset($item);
 	}
 
+	/**
+	 * Check preprocessing steps do not have duplicates with "Check for not supported item" with "error any".
+	 *
+	 * @param array $items
+	 *
+	 * @throws APIException
+	 */
+	protected static function checkPreprocessingStepsDuplicates(array $items): void {
+		$api_input_rules = ['type' => API_OBJECTS, 'fields' => [
+			'preprocessing' => ['type' => API_OBJECTS, 'uniq' => [['type', 'params']], 'fields' => [
+				'type' =>	['type' => API_ANY],
+				'params' =>	['type' => API_ANY]
+			]]
+		]];
+		$items_steps = [];
+
+		foreach ($items as $i1 => $item) {
+			if (!array_key_exists('preprocessing', $item)) {
+				continue;
+			}
+
+			foreach ($item['preprocessing'] as $i2 => $step) {
+				if ($step['type'] == ZBX_PREPROC_VALIDATE_NOT_SUPPORTED) {
+					[$match_type] = explode("\n", $step['params']);
+
+					if ($match_type == ZBX_PREPROC_MATCH_ERROR_ANY) {
+						$items_steps[$i1]['preprocessing'][$i2] = [
+							'type' => ZBX_PREPROC_VALIDATE_NOT_SUPPORTED,
+							'params' => ZBX_PREPROC_MATCH_ERROR_ANY
+						];
+					}
+				}
+			}
+		}
+
+		if (!CApiInputValidator::validateUniqueness($api_input_rules, $items_steps, '', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+	}
+
 	/**
 	 * @param array $items
 	 */
@@ -310,7 +361,7 @@ abstract class CItemGeneral extends CApiService {
 	 * @return array
 	 */
 	protected static function getTagsValidationRules(): array {
-		return ['type' => API_OBJECTS, 'uniq' => [['tag', 'value']], 'fields' => [
+		return ['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['tag', 'value']], 'fields' => [
 			'tag' =>	['type' => API_STRING_UTF8, 'flags' => API_REQUIRED | API_NOT_EMPTY, 'length' => DB::getFieldLength('item_tag', 'tag')],
 			'value' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('item_tag', 'value')]
 		]];
@@ -324,11 +375,11 @@ abstract class CItemGeneral extends CApiService {
 	public static function getPreprocessingValidationRules(int $flags = 0x00): array {
 		return [
 			'type' => API_OBJECTS,
+			'flags' => API_NORMALIZE,
 			'uniq_by_values' => [
 				['type' => [ZBX_PREPROC_DELTA_VALUE, ZBX_PREPROC_DELTA_SPEED]],
 				['type' => [ZBX_PREPROC_THROTTLE_VALUE, ZBX_PREPROC_THROTTLE_TIMED_VALUE]],
-				['type' => [ZBX_PREPROC_PROMETHEUS_PATTERN, ZBX_PREPROC_PROMETHEUS_TO_JSON]],
-				['type' => [ZBX_PREPROC_VALIDATE_NOT_SUPPORTED]]
+				['type' => [ZBX_PREPROC_PROMETHEUS_PATTERN, ZBX_PREPROC_PROMETHEUS_TO_JSON]]
 			],
 			'fields' => [
 				'type' =>					['type' => API_INT32, 'flags' => API_REQUIRED, 'in' => implode(',', static::SUPPORTED_PREPROCESSING_TYPES)],
@@ -392,7 +443,7 @@ abstract class CItemGeneral extends CApiService {
 	}
 
 	/**
-	 * Add host_status property to given items in accordance of given hosts and templates statuses.
+	 * Add host_status property to given items in accordance to statuses of given hosts and templates.
 	 *
 	 * @param array $items
 	 * @param array $db_hosts
@@ -400,12 +451,9 @@ abstract class CItemGeneral extends CApiService {
 	 */
 	protected static function addHostStatus(array &$items, array $db_hosts, array $db_templates): void {
 		foreach ($items as &$item) {
-			if (array_key_exists($item['hostid'], $db_templates)) {
-				$item['host_status'] = HOST_STATUS_TEMPLATE;
-			}
-			else {
-				$item['host_status'] = $db_hosts[$item['hostid']]['status'];
-			}
+			$item['host_status'] = array_key_exists($item['hostid'], $db_templates)
+				? HOST_STATUS_TEMPLATE
+				: $db_hosts[$item['hostid']]['status'];
 		}
 		unset($item);
 	}
@@ -481,6 +529,12 @@ abstract class CItemGeneral extends CApiService {
 					);
 					break;
 
+				case ZBX_FLAG_DISCOVERY_RULE:
+					$error = _s('Invalid parameter "%1$s": %2$s.', '/'.($item_indexes[$duplicates[0]['uuid']] + 1),
+						_('LLD rule with the same UUID already exists')
+					);
+					break;
+
 				case ZBX_FLAG_DISCOVERY_PROTOTYPE:
 					$error = _s('Invalid parameter "%1$s": %2$s.', '/'.($item_indexes[$duplicates[0]['uuid']] + 1),
 						_('item prototype with the same UUID already exists')
@@ -627,8 +681,8 @@ abstract class CItemGeneral extends CApiService {
 
 						case ZBX_FLAG_DISCOVERY_RULE:
 							$error = $target_is_host
-								? _('Cannot inherit LDD rules with key "%1$s" of both "%2$s" and "%3$s" templates, because the key must be unique on host "%4$s".')
-								: _('Cannot inherit LDD rules with key "%1$s" of both "%2$s" and "%3$s" templates, because the key must be unique on template "%4$s".');
+								? _('Cannot inherit LLD rules with key "%1$s" of both "%2$s" and "%3$s" templates, because the key must be unique on host "%4$s".')
+								: _('Cannot inherit LLD rules with key "%1$s" of both "%2$s" and "%3$s" templates, because the key must be unique on template "%4$s".');
 							break;
 					}
 
@@ -662,11 +716,11 @@ abstract class CItemGeneral extends CApiService {
 		$last = 0;
 
 		foreach ($items as $i => $item) {
-			$hosts_chunks = array_chunk($tpl_links[$item['hostid']], self::INHERIT_CHUNK_SIZE, true);
+			$hosts_chunks = array_chunk($tpl_links[$item['hostid']], self::CHUNK_SIZE, true);
 
 			foreach ($hosts_chunks as $hosts) {
-				if ($chunks[$last]['size'] < self::INHERIT_CHUNK_SIZE) {
-					$_hosts = array_slice($hosts, 0, self::INHERIT_CHUNK_SIZE - $chunks[$last]['size'], true);
+				if ($chunks[$last]['size'] < self::CHUNK_SIZE) {
+					$_hosts = array_slice($hosts, 0, self::CHUNK_SIZE - $chunks[$last]['size'], true);
 
 					$can_add_hosts = !array_intersect_key($chunks[$last]['hosts'],
 						array_diff_key($tpl_links[$item['hostid']], $_hosts)
@@ -816,14 +870,14 @@ abstract class CItemGeneral extends CApiService {
 
 			case ZBX_FLAG_DISCOVERY_RULE:
 				$error = $target_is_host
-					? _('Cannot inherit LLD rule with key "%1$s" of template "%2$s" to host "%3$s", because an item with the same key is already inherited from template "%4$s".')
-					: _('Cannot inherit LLD rule with key "%1$s" of template "%2$s" to template "%3$s", because an item with the same key is already inherited from template "%4$s".');
+					? _('Cannot inherit LLD rule with key "%1$s" of template "%2$s" to host "%3$s", because an LLD rule with the same key is already inherited from template "%4$s".')
+					: _('Cannot inherit LLD rule with key "%1$s" of template "%2$s" to template "%3$s", because an LLD rule with the same key is already inherited from template "%4$s".');
 				break;
 
 			case ZBX_FLAG_DISCOVERY_PROTOTYPE:
 				$error = $target_is_host
-					? _('Cannot inherit item prototype with key "%1$s" of template "%2$s" to host "%3$s", because an item with the same key is already inherited from template "%4$s".')
-					: _('Cannot inherit item prototype with key "%1$s" of template "%2$s" to template "%3$s", because an item with the same key is already inherited from template "%4$s".');
+					? _('Cannot inherit item prototype with key "%1$s" of template "%2$s" to host "%3$s", because an item prototype with the same key is already inherited from template "%4$s".')
+					: _('Cannot inherit item prototype with key "%1$s" of template "%2$s" to template "%3$s", because an item prototype with the same key is already inherited from template "%4$s".');
 				break;
 		}
 
@@ -1171,7 +1225,7 @@ abstract class CItemGeneral extends CApiService {
 				$error = _('Cannot inherit item with key "%1$s" of template "%2$s" to host "%3$s", because a host interface of type "%4$s" is required.');
 				break;
 
-			case ZBX_FLAG_DISCOVERY_CREATED:
+			case ZBX_FLAG_DISCOVERY_RULE:
 				$error = _('Cannot inherit LLD rule with key "%1$s" of template "%2$s" to host "%3$s", because a host interface of type "%4$s" is required.');
 				break;
 
@@ -1242,11 +1296,11 @@ abstract class CItemGeneral extends CApiService {
 
 			// HTTP Agent item type specific fields.
 			'url' => DB::getDefault('items', 'url'),
-			'query_fields' => DB::getDefault('items', 'query_fields'),
+			'query_fields' => [],
 			'request_method' => DB::getDefault('items', 'request_method'),
 			'post_type' => DB::getDefault('items', 'post_type'),
 			'posts' => DB::getDefault('items', 'posts'),
-			'headers' => DB::getDefault('items', 'headers'),
+			'headers' => [],
 			'status_codes' => DB::getDefault('items', 'status_codes'),
 			'follow_redirects' => DB::getDefault('items', 'follow_redirects'),
 			'retrieve_mode' => DB::getDefault('items', 'retrieve_mode'),
@@ -1297,47 +1351,83 @@ abstract class CItemGeneral extends CApiService {
 
 				$field_names = array_flip(array_diff($db_type_field_names, $type_field_names));
 
-				if ($item['type'] == ITEM_TYPE_ZABBIX_ACTIVE && strncmp($item['key_'], 'mqtt.get', 8) == 0) {
-					$field_names += array_flip(['delay']);
-				}
-
-				if (array_intersect([$item['type'], $db_item['type']], [ITEM_TYPE_SSH, ITEM_TYPE_HTTPAGENT])) {
-					$field_names += array_flip(['authtype']);
-				}
-
 				if ($item['host_status'] == HOST_STATUS_TEMPLATE && array_key_exists('interfaceid', $field_names)) {
 					unset($field_names['interfaceid']);
 				}
 
 				$item += array_intersect_key($type_field_defaults, $field_names);
 			}
-			elseif ($item['type'] == ITEM_TYPE_ZABBIX_ACTIVE) {
-				if (array_key_exists('key_', $item) && $item['key_'] !== $db_item['key_']
-						&& strncmp($item['key_'], 'mqtt.get', 8) == 0) {
-					$item += array_intersect_key($type_field_defaults, array_flip(['delay']));
-				}
-			}
-			elseif ($item['type'] == ITEM_TYPE_SSH) {
-				if (array_key_exists('authtype', $item) && $item['authtype'] !== $db_item['authtype']
-						&& $item['authtype'] == ITEM_AUTHTYPE_PASSWORD) {
-					$item += array_intersect_key($type_field_defaults, array_flip(['publickey', 'privatekey']));
-				}
-			}
-			elseif ($item['type'] == ITEM_TYPE_HTTPAGENT) {
-				if (array_key_exists('request_method', $item) && $item['request_method'] != $db_item['request_method']
-						&& $item['request_method'] == HTTPCHECK_REQUEST_HEAD) {
-					$item += ['retrieve_mode' => HTTPTEST_STEP_RETRIEVE_MODE_HEADERS];
-				}
 
-				if (array_key_exists('authtype', $item) && $item['authtype'] != $db_item['authtype']
-						&& $item['authtype'] == ZBX_HTTP_AUTH_NONE) {
-					$item += array_intersect_key($type_field_defaults, array_flip(['username', 'password']));
-				}
+			switch ($item['type']) {
+				case ITEM_TYPE_SIMPLE:
+					if (($item['type'] != $db_item['type'] || $item['key_'] !== $db_item['key_'])
+							&& (strncmp($item['key_'], 'icmpping', 8) == 0
+								|| strncmp($item['key_'], 'vmware.', 7) == 0)) {
+						$item += array_intersect_key($type_field_defaults, array_flip(['timeout']));
+					}
+					break;
 
-				if (array_key_exists('allow_traps', $item) && $item['allow_traps'] != $db_item['allow_traps']
-						&& $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF) {
-					$item += array_intersect_key($type_field_defaults, array_flip(['trapper_hosts']));
-				}
+				case ITEM_TYPE_ZABBIX_ACTIVE:
+					if (($item['type'] != $db_item['type'] || $item['key_'] !== $db_item['key_'])
+							&& strncmp($item['key_'], 'mqtt.get', 8) == 0) {
+						$item += array_intersect_key($type_field_defaults, array_flip(['delay']));
+					}
+					break;
+
+				case ITEM_TYPE_SSH:
+					if ($item['type'] != $db_item['type']) {
+						if ($db_item['type'] == ITEM_TYPE_HTTPAGENT) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['authtype']));
+						}
+					}
+					elseif (array_key_exists('authtype', $item) && $item['authtype'] !== $db_item['authtype']
+							&& $item['authtype'] == ITEM_AUTHTYPE_PASSWORD) {
+						$item += array_intersect_key($type_field_defaults, array_flip(['publickey', 'privatekey']));
+					}
+					break;
+
+				case ITEM_TYPE_HTTPAGENT:
+					if ($item['type'] != $db_item['type']) {
+						if (!array_key_exists('authtype', $item)) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['authtype']));
+						}
+
+						if ($item['authtype'] == ZBX_HTTP_AUTH_NONE) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['username', 'password']));
+						}
+
+						if (!array_key_exists('allow_traps', $item)
+								|| $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['trapper_hosts']));
+						}
+					}
+					else {
+						if (array_key_exists('request_method', $item)
+								&& $item['request_method'] != $db_item['request_method']
+								&& $item['request_method'] == HTTPCHECK_REQUEST_HEAD) {
+							$item += ['retrieve_mode' => HTTPTEST_STEP_RETRIEVE_MODE_HEADERS];
+						}
+
+						if (array_key_exists('authtype', $item) && $item['authtype'] != $db_item['authtype']
+								&& $item['authtype'] == ZBX_HTTP_AUTH_NONE) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['username', 'password']));
+						}
+
+						if (array_key_exists('allow_traps', $item) && $item['allow_traps'] != $db_item['allow_traps']
+								&& $item['allow_traps'] == HTTPCHECK_ALLOW_TRAPS_OFF) {
+							$item += array_intersect_key($type_field_defaults, array_flip(['trapper_hosts']));
+						}
+					}
+					break;
+
+				case ITEM_TYPE_SNMP:
+					if (array_key_exists('snmp_oid', $item)
+							&& ($item['type'] != $db_item['type'] || $item['snmp_oid'] !== $db_item['snmp_oid'])
+							&& strncmp($item['snmp_oid'], 'get[', 4) != 0
+							&& strncmp($item['snmp_oid'], 'walk[', 5) != 0) {
+						$item += array_intersect_key($type_field_defaults, array_flip(['timeout']));
+					}
+					break;
 			}
 
 			if (array_key_exists('value_type', $item) && $item['value_type'] != $db_item['value_type']) {
@@ -1361,7 +1451,7 @@ abstract class CItemGeneral extends CApiService {
 	 * @param array|null $db_items
 	 * @param array|null $upd_itemids
 	 */
-	protected static function updateParameters(array &$items, array $db_items = null,
+	protected static function updateParameters(array &$items, array &$db_items = null,
 			array &$upd_itemids = null): void {
 		$ins_item_parameters = [];
 		$upd_item_parameters = [];
@@ -1371,7 +1461,8 @@ abstract class CItemGeneral extends CApiService {
 			$update = false;
 
 			if ($db_items === null) {
-				if ($item['type'] == ITEM_TYPE_SCRIPT && array_key_exists('parameters', $item) && $item['parameters']) {
+				if (in_array($item['type'], [ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER])
+						&& array_key_exists('parameters', $item) && $item['parameters']) {
 					$update = true;
 				}
 			}
@@ -1380,12 +1471,10 @@ abstract class CItemGeneral extends CApiService {
 					continue;
 				}
 
-				if ($item['type'] == ITEM_TYPE_SCRIPT) {
-					if (array_key_exists('parameters', $item)) {
-						$update = true;
-					}
+				if (in_array($item['type'], [ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER])) {
+					$update = array_key_exists('parameters', $item);
 				}
-				elseif ($db_items[$item['itemid']]['type'] == ITEM_TYPE_SCRIPT
+				elseif (in_array($db_items[$item['itemid']]['type'], [ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER])
 						&& $db_items[$item['itemid']]['parameters']) {
 					$update = true;
 				}
@@ -1434,7 +1523,7 @@ abstract class CItemGeneral extends CApiService {
 					$upd_itemids[$i] = $item['itemid'];
 				}
 				else {
-					unset($item['parameters']);
+					unset($item['parameters'], $db_items[$item['itemid']]['parameters']);
 				}
 			}
 		}
@@ -1472,7 +1561,7 @@ abstract class CItemGeneral extends CApiService {
 	 * @param array|null $db_items
 	 * @param array|null $upd_itemids
 	 */
-	protected static function updatePreprocessing(array &$items, array $db_items = null,
+	protected static function updatePreprocessing(array &$items, array &$db_items = null,
 			array &$upd_itemids = null): void {
 		$ins_item_preprocs = [];
 		$upd_item_preprocs = [];
@@ -1489,9 +1578,10 @@ abstract class CItemGeneral extends CApiService {
 				: [];
 
 			$step = 1;
+			$item['preprocessing'] = self::sortPreprocessingSteps($item['preprocessing']);
 
 			foreach ($item['preprocessing'] as &$item_preproc) {
-				$item_preproc['step'] = ($item_preproc['type'] == ZBX_PREPROC_VALIDATE_NOT_SUPPORTED) ? 0 : $step++;
+				$item_preproc['step'] = $step++;
 
 				if (array_key_exists($item_preproc['step'], $db_item_preprocs)) {
 					$db_item_preproc = $db_item_preprocs[$item_preproc['step']];
@@ -1526,7 +1616,7 @@ abstract class CItemGeneral extends CApiService {
 					$upd_itemids[$i] = $item['itemid'];
 				}
 				else {
-					unset($item['preprocessing']);
+					unset($item['preprocessing'], $db_items[$item['itemid']]['preprocessing']);
 				}
 			}
 		}
@@ -1564,7 +1654,7 @@ abstract class CItemGeneral extends CApiService {
 	 * @param array|null $db_items
 	 * @param array|null $upd_itemids
 	 */
-	protected static function updateTags(array &$items, array $db_items = null, array &$upd_itemids = null): void {
+	protected static function updateTags(array &$items, array &$db_items = null, array &$upd_itemids = null): void {
 		$ins_tags = [];
 		$del_itemtagids = [];
 
@@ -1603,7 +1693,7 @@ abstract class CItemGeneral extends CApiService {
 					$upd_itemids[$i] = $item['itemid'];
 				}
 				else {
-					unset($item['tags']);
+					unset($item['tags'], $db_items[$item['itemid']]['tags']);
 				}
 			}
 		}
@@ -2155,6 +2245,10 @@ abstract class CItemGeneral extends CApiService {
 	 */
 	private static function checkCircularDependencies(array $items, array $dep_item_links): void {
 		foreach ($items as $i => $item) {
+			if ($item['flags'] == ZBX_FLAG_DISCOVERY_RULE) {
+				continue;
+			}
+
 			$master_itemid = $item['master_itemid'];
 
 			while ($master_itemid != 0) {
@@ -2169,53 +2263,6 @@ abstract class CItemGeneral extends CApiService {
 		}
 	}
 
-	/**
-	 * Check prerpocessing steps for specifics validation rules.
-	 *
-	 * @param array $items
-	 *
-	 * @throws APIException
-	 */
-	protected static function checkPreprocessingSteps(array $items): void {
-		foreach ($items as $i => $item) {
-			if (!array_key_exists('preprocessing', $item)) {
-				continue;
-			}
-
-			foreach ($item['preprocessing'] as $j => $step) {
-				if ($step['type'] == ZBX_PREPROC_SNMP_WALK_TO_JSON) {
-					$params = explode("\n", $step['params']);
-
-					if (count($params) % 3 !== 0) {
-						self::exception(ZBX_API_ERROR_PARAMETERS,
-							_s('Incorrect value for field "%1$s": %2$s.', '/'.($i + 1).'/preprocessing/'.($j + 1).'/params', _('cannot be empty'))
-						);
-					}
-
-					for ($n = 1; $n <= count($params); $n++) {
-						$param = $params[$n - 1];
-
-						if ($param === '') {
-							self::exception(ZBX_API_ERROR_PARAMETERS,
-								_s('Incorrect value for field "%1$s": %2$s.', '/'.($i + 1).'/preprocessing/'.($j + 1).'/params', _('cannot be empty'))
-							);
-						}
-
-						// Field "Treat as" every 3rd value. Check that field is correct.
-						if ($n % 3 === 0) {
-							if (!in_array($param, [ZBX_PREPROC_SNMP_UNCHANGED, ZBX_PREPROC_SNMP_UTF8_FROM_HEX,
-									ZBX_PREPROC_SNMP_MAC_FROM_HEX, ZBX_PREPROC_SNMP_INT_FROM_BITS])) {
-								self::exception(ZBX_API_ERROR_PARAMETERS,
-									_s('Incorrect value for field "%1$s": %2$s.', '/'.($i + 1).'/preprocessing/'.($j + 1).'/params', _('incorrect value'))
-								);
-							}
-						}
-					}
-				}
-			}
-		}
-	}
-
 	/**
 	 * Get master item links starting from the given master items and till the lowest level master items.
 	 *
@@ -2430,44 +2477,6 @@ abstract class CItemGeneral extends CApiService {
 		}
 	}
 
-	/**
-	 * Converts headers field text to hash with header name as key.
-	 *
-	 * @param string $headers  Headers string, one header per line, line delimiter "\r\n".
-	 *
-	 * @return array
-	 */
-	protected static function headersStringToArray(string $headers): array {
-		$result = [];
-
-		foreach (explode("\r\n", $headers) as $header) {
-			$header = explode(': ', $header, 2);
-
-			if (count($header) == 2) {
-				$result[$header[0]] = $header[1];
-			}
-		}
-
-		return $result;
-	}
-
-	/**
-	 * Converts headers fields hash to string.
-	 *
-	 * @param array $headers  Array of headers where key is header name.
-	 *
-	 * @return string
-	 */
-	protected static function headersArrayToString(array $headers): string {
-		$result = [];
-
-		foreach ($headers as $k => $v) {
-			$result[] = $k.': '.$v;
-		}
-
-		return implode("\r\n", $result);
-	}
-
 	/**
 	 * Remove NCLOB value type fields from resulting query SELECT part if DISTINCT will be used.
 	 *
@@ -2479,66 +2488,11 @@ abstract class CItemGeneral extends CApiService {
 	 * @return array    The resulting SQL parts array.
 	 */
 	protected function applyQueryOutputOptions($table_name, $table_alias, array $options, array $sql_parts) {
-		if (!$options['countOutput'] && self::dbDistinct($sql_parts)) {
-			$schema = $this->getTableSchema();
-			$nclob_fields = [];
-
-			foreach ($schema['fields'] as $field_name => $field) {
-				if ($field['type'] == DB::FIELD_TYPE_NCLOB
-						&& $this->outputIsRequested($field_name, $options['output'])) {
-					$nclob_fields[] = $field_name;
-				}
-			}
-
-			if ($nclob_fields) {
-				$output = ($options['output'] === API_OUTPUT_EXTEND)
-					? array_keys($schema['fields'])
-					: $options['output'];
-
-				$options['output'] = array_diff($output, $nclob_fields);
-			}
-		}
+		$this->unsetNclobFieldsFromOutput($options, $sql_parts);
 
 		return parent::applyQueryOutputOptions($table_name, $table_alias, $options, $sql_parts);
 	}
 
-	/**
-	 * Add NCLOB type fields if there was DISTINCT in query.
-	 *
-	 * @param array $options    Array of query options.
-	 * @param array $result     Query results.
-	 *
-	 * @return array    The result array with added NCLOB fields.
-	 */
-	protected function addNclobFieldValues(array $options, array $result): array {
-		$schema = $this->getTableSchema();
-		$nclob_fields = [];
-
-		foreach ($schema['fields'] as $field_name => $field) {
-			if ($field['type'] == DB::FIELD_TYPE_NCLOB && $this->outputIsRequested($field_name, $options['output'])) {
-				$nclob_fields[] = $field_name;
-			}
-		}
-
-		if (!$nclob_fields) {
-			return $result;
-		}
-
-		$pk = $schema['key'];
-		$options = [
-			'output' => $nclob_fields,
-			'filter' => [$pk => array_keys($result)]
-		];
-
-		$db_items = DBselect(DB::makeSql($this->tableName, $options));
-
-		while ($db_item = DBfetch($db_items)) {
-			$result[$db_item[$pk]] += $db_item;
-		}
-
-		return $result;
-	}
-
 	/**
 	 * Check that valuemap belong to same host as item.
 	 *
@@ -2661,7 +2615,8 @@ abstract class CItemGeneral extends CApiService {
 			'output' => [
 				'item_preprocid', 'itemid', 'step', 'type', 'params', 'error_handler', 'error_handler_params'
 			],
-			'filter' => ['itemid' => $itemids]
+			'filter' => ['itemid' => $itemids],
+			'sortfield' => ['itemid', 'step']
 		];
 		$db_item_preprocs = DBselect(DB::makeSql('item_preproc', $options));
 
@@ -2679,10 +2634,11 @@ abstract class CItemGeneral extends CApiService {
 		$itemids = [];
 
 		foreach ($items as $item) {
+			$type = $item['type'];
 			$db_type = $db_items[$item['itemid']]['type'];
 
-			if ((array_key_exists('parameters', $item) && $item['type'] == ITEM_TYPE_SCRIPT)
-					|| ($item['type'] != $db_type && $db_type == ITEM_TYPE_SCRIPT)) {
+			if ((array_key_exists('parameters', $item) && in_array($type, [ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER]))
+					|| ($type != $db_type && in_array($db_type, [ITEM_TYPE_SCRIPT, ITEM_TYPE_BROWSER]))) {
 				$itemids[] = $item['itemid'];
 				$db_items[$item['itemid']]['parameters'] = [];
 			}
@@ -2790,4 +2746,125 @@ abstract class CItemGeneral extends CApiService {
 			CTriggerPrototypeManager::delete($del_trigger_prototypeids);
 		}
 	}
+
+	/**
+	 * Prioritize ZBX_PREPROC_VALIDATE_NOT_SUPPORTED checks, with "match any error" being the last of them.
+	 *
+	 * @param array $steps
+	 *
+	 * @return array
+	 */
+	private static function sortPreprocessingSteps(array $steps): array {
+		$ns_regex = [];
+		$ns_any = [];
+		$other = [];
+
+		foreach ($steps as $step) {
+			if ($step['type'] != ZBX_PREPROC_VALIDATE_NOT_SUPPORTED) {
+				$other[] = $step;
+				continue;
+			}
+
+			[$match_type] = explode("\n", $step['params']);
+
+			if ($match_type == ZBX_PREPROC_MATCH_ERROR_ANY) {
+				$ns_any[] = $step;
+			}
+			else {
+				$ns_regex[] = $step;
+			}
+		}
+
+		return array_merge($ns_regex, $ns_any, $other);
+	}
+
+	protected static function prepareItemsForApi(array &$items, bool $sortorder = true): void {
+		foreach ($items as &$item) {
+			self::prepareItemForApi($item, $sortorder);
+		}
+		unset($item);
+	}
+
+	protected static function prepareItemForApi(array &$item, bool $sortorder = true): void {
+		if (array_key_exists('query_fields', $item)) {
+			$item['query_fields'] = self::prepareQueryFieldsForApi($item['query_fields'], $sortorder);
+		}
+
+		if (array_key_exists('headers', $item)) {
+			$item['headers'] = self::prepareHeadersForApi($item['headers'], $sortorder);
+		}
+	}
+
+	private static function prepareQueryFieldsForApi(string $query_fields, bool $sortorder): array {
+		if ($query_fields === '') {
+			return [];
+		}
+
+		$_query_fields = json_decode($query_fields, true);
+		$query_fields = [];
+
+		if (json_last_error() == JSON_ERROR_NONE) {
+			foreach ($_query_fields as $i => $field) {
+				$query_fields[] = $sortorder
+					? ['sortorder' => $i + 1, 'name' => (string) key($field), 'value' => reset($field)]
+					: ['name' => (string) key($field), 'value' => reset($field)];
+			}
+		}
+
+		return $query_fields;
+	}
+
+	private static function prepareHeadersForApi(string $headers, bool $sortorder): array {
+		if ($headers === '') {
+			return [];
+		}
+
+		$_headers = explode("\r\n", $headers);
+		$headers = [];
+
+		foreach ($_headers as $i => $header) {
+			[$name, $value] = explode(': ', $header, 2) + [1 => ''];
+
+			$headers[] = $sortorder
+				? ['sortorder' => $i + 1, 'name' => $name, 'value' => $value]
+				: ['name' => $name, 'value' => $value];
+		}
+
+		return $headers;
+	}
+
+	protected static function prepareItemsForDb(array &$items): void {
+		foreach ($items as &$item) {
+			self::prepareItemForDb($item);
+		}
+		unset($item);
+	}
+
+	private static function prepareItemForDb(array &$item): void {
+		if (array_key_exists('query_fields', $item)) {
+			$item['query_fields'] = self::prepareQueryFieldsForDb($item['query_fields']);
+		}
+
+		if (array_key_exists('headers', $item)) {
+			$item['headers'] = self::prepareHeadersForDb($item['headers']);
+		}
+	}
+
+	private static function prepareQueryFieldsForDb(array $query_fields): string {
+		foreach ($query_fields as &$query_field) {
+			$query_field = [$query_field['name'] => $query_field['value']];
+		}
+		unset($query_field);
+
+		return $query_fields ? json_encode(array_values($query_fields), JSON_UNESCAPED_UNICODE) : '';
+	}
+
+	private static function prepareHeadersForDb(array $headers): string {
+		foreach ($headers as &$header) {
+			$header = $header['name'].': '.$header['value'];
+		}
+		unset($header);
+
+		return $headers ? implode("\r\n", $headers) : '';
+	}
 }
