@@ -1,21 +1,16 @@
 <?php declare(strict_types = 0);
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -28,8 +23,8 @@ class CSvgGraph extends CSvg {
 	public const SVG_GRAPH_DEFAULT_LINE_WIDTH = 1;
 
 	public const SVG_GRAPH_X_AXIS_LABEL_MARGIN = 5;
-	public const SVG_GRAPH_Y_AXIS_LEFT_LABEL_MARGIN = 5;
-	public const SVG_GRAPH_Y_AXIS_RIGHT_LABEL_MARGIN = 12;
+	public const SVG_GRAPH_Y_AXIS_LABEL_MARGIN_OUTER = 10;
+	public const SVG_GRAPH_Y_AXIS_LABEL_MARGIN_INNER = 5;
 
 	private $canvas_x;
 	private $canvas_y;
@@ -103,7 +98,6 @@ class CSvgGraph extends CSvg {
 	private $left_y_max_calculated;
 	private $left_y_interval;
 	private $left_y_units;
-	private $left_y_is_binary;
 	private $left_y_power;
 	private $left_y_empty = true;
 	private $left_y_zero;
@@ -115,7 +109,6 @@ class CSvgGraph extends CSvg {
 	private $right_y_max_calculated;
 	private $right_y_interval;
 	private $right_y_units;
-	private $right_y_is_binary;
 	private $right_y_power;
 	private $right_y_empty = true;
 	private $right_y_zero;
@@ -736,7 +729,7 @@ class CSvgGraph extends CSvg {
 
 		$offset_top = 10;
 		$offset_bottom = self::SVG_GRAPH_X_AXIS_HEIGHT;
-		$this->canvas_height = $this->height - $offset_top - $offset_bottom;
+		$this->canvas_height = max(0, $this->height - $offset_top - $offset_bottom);
 		$this->canvas_y = $offset_top;
 
 		// Determine units for left side.
@@ -778,12 +771,10 @@ class CSvgGraph extends CSvg {
 			$this->left_y_max = $this->max_value_left ?: 1;
 		}
 
-		$this->left_y_is_binary = isBinaryUnits($this->left_y_units);
-
 		$calc_power = $this->left_y_units === '' || $this->left_y_units[0] !== '!';
 
-		$result = calculateGraphScaleExtremes($this->left_y_min, $this->left_y_max, $this->left_y_is_binary,
-			$calc_power, $this->left_y_min_calculated, $this->left_y_max_calculated, $rows_min, $rows_max
+		$result = calculateGraphScaleExtremes($this->left_y_min, $this->left_y_max, $this->left_y_units, $calc_power,
+			$this->left_y_min_calculated, $this->left_y_max_calculated, $rows_min, $rows_max
 		);
 
 		[
@@ -795,8 +786,9 @@ class CSvgGraph extends CSvg {
 
 		// Calculate vertical scale parameters for right side.
 
-		if ($this->left_y_min_calculated && $this->left_y_max_calculated) {
-			$rows_min = $rows_max = $result['rows'];
+		if ($this->show_left_y_axis && $this->left_y_min_calculated && $this->left_y_max_calculated) {
+			$rows_min = $result['rows'];
+			$rows_max = $result['rows'];
 		}
 
 		$this->right_y_min_calculated = $this->right_y_min === null;
@@ -809,12 +801,10 @@ class CSvgGraph extends CSvg {
 			$this->right_y_max = $this->max_value_right ?: 1;
 		}
 
-		$this->right_y_is_binary = isBinaryUnits($this->right_y_units);
-
 		$calc_power = $this->right_y_units === '' || $this->right_y_units[0] !== '!';
 
-		$result = calculateGraphScaleExtremes($this->right_y_min, $this->right_y_max, $this->right_y_is_binary,
-			$calc_power, $this->right_y_min_calculated, $this->right_y_max_calculated, $rows_min, $rows_max
+		$result = calculateGraphScaleExtremes($this->right_y_min, $this->right_y_max, $this->right_y_units, $calc_power,
+			$this->right_y_min_calculated, $this->right_y_max_calculated, $rows_min, $rows_max
 		);
 
 		[
@@ -825,15 +815,19 @@ class CSvgGraph extends CSvg {
 		] = $result;
 
 		// Define canvas dimensions and offsets, except canvas height and bottom offset.
-
-		$approx_width = 10;
-
 		if ($this->show_left_y_axis) {
 			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_LEFT, $this->left_y_empty);
 
 			if ($values) {
-				$offset_left = max($this->offset_left, max(array_map('strlen', $values)) * $approx_width);
-				$this->offset_left = (int) min($offset_left, $this->max_yaxis_width);
+				$approx_width = 0;
+
+				foreach ($values as $value) {
+					$approx_width = max($approx_width, imageTextSize(11, 0, $value)['width']);
+				}
+
+				$this->offset_left = min($this->max_yaxis_width,
+					max($this->offset_left, self::SVG_GRAPH_Y_AXIS_LABEL_MARGIN_OUTER + $approx_width)
+				);
 			}
 		}
 
@@ -841,13 +835,23 @@ class CSvgGraph extends CSvg {
 			$values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT, $this->right_y_empty);
 
 			if ($values) {
-				$offset_right = max($this->offset_right, max(array_map('strlen', $values)) * $approx_width);
-				$offset_right += self::SVG_GRAPH_Y_AXIS_RIGHT_LABEL_MARGIN;
-				$this->offset_right = (int) min($offset_right, $this->max_yaxis_width);
+				if (array_key_exists(0, $values)) {
+					unset($values[0]);
+				}
+
+				$approx_width = 0;
+
+				foreach ($values as $value) {
+					$approx_width = max($approx_width, imageTextSize(11, 0, $value)['width']);
+				}
+
+				$this->offset_right = min($this->max_yaxis_width,
+					max($this->offset_right, self::SVG_GRAPH_Y_AXIS_LABEL_MARGIN_OUTER + $approx_width)
+				);
 			}
 		}
 
-		$this->canvas_width = $this->width - $this->offset_left - $this->offset_right;
+		$this->canvas_width = max(0, $this->width - $this->offset_left - $this->offset_right);
 		$this->canvas_x = $this->offset_left;
 
 		// Calculate vertical zero position.
@@ -1202,7 +1206,7 @@ class CSvgGraph extends CSvg {
 		if ($this->show_right_y_axis) {
 			$grid_values = $this->getValuesGridWithPosition(GRAPH_YAXIS_SIDE_RIGHT, $this->right_y_empty);
 
-			// Do not draw label at the bottom of right Y axis to avoid label averlapping with horizontal axis arrow.
+			// Do not draw label at the bottom of right Y axis to avoid label overlapping with horizontal axis arrow.
 			if (array_key_exists(0, $grid_values)) {
 				unset($grid_values[0]);
 			}
@@ -1623,7 +1627,6 @@ class CSvgGraph extends CSvg {
 		$max_calculated = true;
 		$interval = 1;
 		$units = '';
-		$is_binary = false;
 		$power = 0;
 
 		if (!$empty_set) {
@@ -1634,7 +1637,6 @@ class CSvgGraph extends CSvg {
 				$max_calculated = $this->left_y_max_calculated;
 				$interval = $this->left_y_interval;
 				$units = $this->left_y_units;
-				$is_binary = $this->left_y_is_binary;
 				$power = $this->left_y_power;
 			}
 			elseif ($side === GRAPH_YAXIS_SIDE_RIGHT) {
@@ -1644,13 +1646,12 @@ class CSvgGraph extends CSvg {
 				$max_calculated = $this->right_y_max_calculated;
 				$interval = $this->right_y_interval;
 				$units = $this->right_y_units;
-				$is_binary = $this->right_y_is_binary;
 				$power = $this->right_y_power;
 			}
 		}
 
 		$relative_values = calculateGraphScaleValues($min, $max, $min_calculated, $max_calculated, $interval, $units,
-			$is_binary, $power, 14
+			$power, 14
 		);
 
 		$absolute_values = [];
