@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -712,14 +707,7 @@ function getSelementsInfo(array $sysmap, array $options = []): array {
 			unset($trigger);
 
 			// Assign selected hosts back to the sysmap elements they origin from.
-			if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST) {
-				$hostid = $selement['elements'][0]['hostid'];
-				$selement['hosts'][$hostid] = $hostid;
-			}
-			elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST_GROUP) {
-				$groupid = $selement['elements'][0]['groupid'];
-				$selement['hosts'] = $hosts_by_groupids[$groupid];
-			}
+			$selement['hosts'] = getElementHosts($selement, $sysmaps_data, $hosts_by_groupids);
 		}
 		unset($selement);
 	}
@@ -931,6 +919,30 @@ function getSelementsInfo(array $sysmap, array $options = []): array {
 	return $info;
 }
 
+function getElementHosts($selement, $sysmaps_data, $hosts_by_groupids) {
+	$host_ids = [];
+
+	if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST) {
+		$hostid = $selement['elements'][0]['hostid'];
+		$host_ids[$hostid] = $hostid;
+	}
+	elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_HOST_GROUP) {
+		$groupid = $selement['elements'][0]['groupid'];
+		$host_ids = $hosts_by_groupids[$groupid];
+	}
+	elseif ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_MAP) {
+		$sysmapid = $selement['elements'][0]['sysmapid'];
+
+		if (array_key_exists($sysmapid, $sysmaps_data)) {
+			foreach ($sysmaps_data[$sysmapid]['selements'] as $nested_element) {
+				$host_ids += getElementHosts($nested_element, $sysmaps_data, $hosts_by_groupids);
+			}
+		}
+	}
+
+	return $host_ids;
+}
+
 function countSelementProblems(array $selement, array &$sysmaps_data): array {
 	if ($selement['elementtype'] == SYSMAP_ELEMENT_TYPE_MAP) {
 		return countNestedMapSelementProblems($selement, $sysmaps_data);
