@@ -1,26 +1,22 @@
 <?php
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
 /**
  * @var CView $this
+ * @var array $data
  */
 
 require_once dirname(__FILE__).'/js/configuration.host.discovery.list.js.php';
@@ -116,7 +112,7 @@ $filter_type_visibility = [];
 $type_select = (new CSelect('filter_type'))
 	->setId('filter_type')
 	->setFocusableElementId('label-type')
-	->addOption(new CSelectOption(-1, _('all')))
+	->addOption(new CSelectOption(-1, _('All')))
 	->setValue($data['filter']['type']);
 
 zbx_subarray_push($filter_type_visibility, -1, 'filter_delay_row');
@@ -147,35 +143,61 @@ $filter_column2 = (new CFormList())
 		(new CTextBox('filter_delay', $data['filter']['delay']))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
 		'filter_delay_row'
 	)
-	->addRow(_('Keep lost resources period'),
-		(new CTextBox('filter_lifetime', $data['filter']['lifetime']))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH)
+	->addRow(
+		new CLabel(_('Delete lost resources'), 'filter_lifetime'),
+		new CFormField([
+			(new CRadioButtonList('filter_lifetime_type', (int) $data['filter']['lifetime_type']))
+				->addValue(_('All'), -1)
+				->addValue(_('Never'), ZBX_LLD_DELETE_NEVER)
+				->addValue(_('Immediately'), ZBX_LLD_DELETE_IMMEDIATELY)
+				->addValue(_('After'), ZBX_LLD_DELETE_AFTER)
+				->setModern(),
+			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+			(new CTextBox('filter_lifetime', $data['filter']['lifetime']))
+				->setAttribute('disabled', $data['filter']['lifetime_type'] != ZBX_LLD_DELETE_AFTER)
+				->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
+		])
+	)
+	->addRow(
+		new CLabel(_('Disable lost resources'), 'filter_enabled_lifetime'),
+		new CFormField([
+			(new CRadioButtonList('filter_enabled_lifetime_type', (int) $data['filter']['enabled_lifetime_type']))
+				->addValue(_('All'), -1)
+				->addValue(_('Never'), ZBX_LLD_DISABLE_NEVER)
+				->addValue(_('Immediately'), ZBX_LLD_DISABLE_IMMEDIATELY)
+				->addValue(_('After'), ZBX_LLD_DISABLE_AFTER)
+				->setModern(),
+			(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
+			(new CTextBox('filter_enabled_lifetime', $data['filter']['enabled_lifetime']))
+				->setAttribute('disabled', $data['filter']['enabled_lifetime_type'] != ZBX_LLD_DISABLE_AFTER)
+				->setWidth(ZBX_TEXTAREA_TINY_WIDTH)
+		])
 	)
 	->addRow(_('SNMP OID'),
 		(new CTextBox('filter_snmp_oid', $data['filter']['snmp_oid']))->setWidth(ZBX_TEXTAREA_FILTER_SMALL_WIDTH),
 		'filter_snmp_oid_row'
 	);
 
-$filter_column3 = (new CFormList());
-
 if ($data['context'] === 'host') {
-	$filter_column3->addRow(_('State'),
+	$filter_column2->addRow(_('State'),
 		(new CRadioButtonList('filter_state', (int) $data['filter']['state']))
-			->addValue(_('all'), -1)
+			->addValue(_('All'), -1)
 			->addValue(_('Normal'), ITEM_STATE_NORMAL)
 			->addValue(_('Not supported'), ITEM_STATE_NOTSUPPORTED)
 			->setModern(true)
 	);
 }
 
-$filter_column3->addRow(_('Status'),
+$filter_column2->addRow(_('Status'),
 	(new CRadioButtonList('filter_status', (int) $data['filter']['status']))
-		->addValue(_('all'), -1)
+		->addValue(_('All'), -1)
 		->addValue(_('Enabled'), ITEM_STATUS_ACTIVE)
 		->addValue(_('Disabled'), ITEM_STATUS_DISABLED)
+		->setEnabled($data['context'] !== 'host' || $data['filter']['state'] == -1)
 		->setModern(true)
 );
 
-$filter->addFilterTab(_('Filter'), [$filter_column1, $filter_column2, $filter_column3]);
+$filter->addFilterTab(_('Filter'), [$filter_column1, $filter_column2]);
 
 $html_page->addItem($filter);
 
@@ -207,7 +229,8 @@ $discoveryTable = (new CTableInfo())
 		make_sorting_header(_('Type'), 'type', $data['sort'], $data['sortorder'], $url),
 		make_sorting_header(_('Status'), 'status', $data['sort'], $data['sortorder'], $url),
 		($data['context'] === 'host') ? _('Info') : null
-	]);
+	])
+	->setPageNavigation($data['paging']);
 
 $update_interval_parser = new CUpdateIntervalParser(['usermacros' => true]);
 $csrf_token = CCsrfTokenHelper::get('host_discovery.php');
@@ -224,15 +247,12 @@ foreach ($data['discoveries'] as $discovery) {
 			$description[] = $discovery['master_item']['name'];
 		}
 		else {
-			$description[] = (new CLink($discovery['master_item']['name'],
-				(new CUrl('items.php'))
-					->setArgument('form', 'update')
-					->setArgument('itemid', $discovery['master_item']['itemid'])
-					->setArgument('context', $data['context'])
-					->getUrl()
-			))
+			$description[] = (new CLink($discovery['master_item']['name']))
 				->addClass(ZBX_STYLE_LINK_ALT)
-				->addClass(ZBX_STYLE_TEAL);
+				->addClass(ZBX_STYLE_TEAL)
+				->addClass('js-update-item')
+				->setAttribute('data-itemid', $discovery['master_item']['itemid'])
+				->setAttribute('data-context', $data['context']);
 		}
 
 		$description[] = NAME_DELIMITER;
@@ -267,7 +287,7 @@ foreach ($data['discoveries'] as $discovery) {
 	// Hide zeros for trapper, SNMP trap and dependent items.
 	if ($discovery['type'] == ITEM_TYPE_TRAPPER || $discovery['type'] == ITEM_TYPE_SNMPTRAP
 			|| $discovery['type'] == ITEM_TYPE_DEPENDENT || ($discovery['type'] == ITEM_TYPE_ZABBIX_ACTIVE
-				&& strncmp($discovery['key_'], 'mqtt.get', 8) === 0)) {
+				&& strncmp($discovery['key_'], 'mqtt.get', 8) == 0)) {
 		$discovery['delay'] = '';
 	}
 	elseif ($update_interval_parser->parse($discovery['delay']) == CParser::PARSE_SUCCESS) {
@@ -291,13 +311,18 @@ foreach ($data['discoveries'] as $discovery) {
 		$checkbox->setAttribute('data-actions', 'execute');
 	}
 
+	$host = (new CLink($discovery['hosts'][0]['name']))
+		->setAttribute('data-hostid', $discovery['hosts'][0]['hostid'])
+		->addClass('js-edit-'.$data['context']);
+
 	$discoveryTable->addRow([
 		$checkbox,
-		$discovery['hosts'][0]['name'],
-		$description,
+		(new CCol($host))->addClass(ZBX_STYLE_WORDBREAK),
+		(new CCol($description))->addClass(ZBX_STYLE_WORDBREAK),
 		[
 			new CLink(_('Item prototypes'),
-				(new CUrl('disc_prototypes.php'))
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'item.prototype.list')
 					->setArgument('parent_discoveryid', $discovery['itemid'])
 					->setArgument('context', $data['context'])
 			),
@@ -305,7 +330,8 @@ foreach ($data['discoveries'] as $discovery) {
 		],
 		[
 			new CLink(_('Trigger prototypes'),
-				(new CUrl('trigger_prototypes.php'))
+				(new CUrl('zabbix.php'))
+					->setArgument('action', 'trigger.prototype.list')
 					->setArgument('parent_discoveryid', $discovery['itemid'])
 					->setArgument('context', $data['context'])
 			),
@@ -338,10 +364,16 @@ foreach ($data['discoveries'] as $discovery) {
 }
 
 $button_list = [
-	'discoveryrule.massenable' => ['name' => _('Enable'), 'confirm' =>_('Enable selected discovery rules?'),
+	'discoveryrule.massenable' => [
+		'name' => _('Enable'),
+		'confirm_singular' => _('Enable selected discovery rule?'),
+		'confirm_plural' => _('Enable selected discovery rules?'),
 		'csrf_token' => $csrf_token
 	],
-	'discoveryrule.massdisable' => ['name' => _('Disable'), 'confirm' =>_('Disable selected discovery rules?'),
+	'discoveryrule.massdisable' => [
+		'name' => _('Disable'),
+		'confirm_singular' => _('Disable selected discovery rule?'),
+		'confirm_plural' => _('Disable selected discovery rules?'),
 		'csrf_token' => $csrf_token
 	]
 ];
@@ -350,24 +382,28 @@ if ($data['context'] === 'host') {
 	$button_list += [
 		'discoveryrule.masscheck_now' => [
 			'content' => (new CSimpleButton(_('Execute now')))
-				->onClick('view.massCheckNow(this);')
 				->addClass(ZBX_STYLE_BTN_ALT)
-				->addClass('no-chkbxrange')
+				->addClass('js-massexecute-item')
+				->addClass('js-no-chkbxrange')
 				->setAttribute('data-required', 'execute')
 		]
 	];
 }
 
 $button_list += [
-	'discoveryrule.massdelete' => ['name' => _('Delete'), 'confirm' =>_('Delete selected discovery rules?'),
+	'discoveryrule.massdelete' => [
+		'name' => _('Delete'),
+		'confirm_singular' => _('Delete selected discovery rule?'),
+		'confirm_plural' => _('Delete selected discovery rules?'),
 		'csrf_token' => $csrf_token
 	]
 ];
 
 // Append table to form.
-$discoveryForm->addItem([$discoveryTable, $data['paging'], new CActionButtonList('action', 'g_hostdruleid',
-	$button_list, $data['checkbox_hash']
-)]);
+$discoveryForm->addItem([
+	$discoveryTable,
+	new CActionButtonList('action', 'g_hostdruleid', $button_list, $data['checkbox_hash'])
+]);
 
 $html_page
 	->addItem($discoveryForm)
@@ -375,8 +411,11 @@ $html_page
 
 (new CScriptTag('
 	view.init('.json_encode([
+		'context' => $data['context'],
 		'checkbox_hash' => $data['checkbox_hash'],
-		'checkbox_object' => 'g_hostdruleid'
+		'checkbox_object' => 'g_hostdruleid',
+		'token' => [CSRF_TOKEN_NAME, CCsrfTokenHelper::get('item')],
+		'form_name' => $discoveryForm->getName()
 	]).');
 '))
 	->setOnDocumentReady()
