@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -25,14 +20,13 @@
 ?>
 
 <script type="text/x-jquery-tmpl" id="tmpl-item-row-<?= GRAPH_TYPE_NORMAL ?>">
-	<tr id="items_#{number}" class="sortable">
+	<tr id="items_#{number}" class="graph-item">
 		<!-- icon + hidden -->
 		<?php if ($readonly): ?>
 			<td>
 		<?php else: ?>
 			<td class="<?= ZBX_STYLE_TD_DRAG_ICON ?>">
 				<div class="<?= ZBX_STYLE_DRAG_ICON ?>"></div>
-				<span class="ui-icon ui-icon-arrowthick-2-n-s move"></span>
 		<?php endif ?>
 			<input type="hidden" id="items_#{number}_gitemid" name="items[#{number}][gitemid]" value="#{gitemid}">
 			<input type="hidden" id="items_#{number}_itemid" name="items[#{number}][itemid]" value="#{itemid}">
@@ -46,7 +40,7 @@
 
 		<!-- row number -->
 		<td>
-			<span id="items_#{number}_number" class="items_number">#{number_nr}:</span>
+			<span class="<?= ZBX_STYLE_LIST_NUMBERED_ITEM ?>">:</span>
 		</td>
 
 		<!-- name -->
@@ -108,14 +102,13 @@
 </script>
 
 <script type="text/x-jquery-tmpl" id="tmpl-item-row-<?= GRAPH_TYPE_STACKED ?>">
-	<tr id="items_#{number}" class="sortable">
+	<tr id="items_#{number}" class="graph-item">
 		<!-- icon + hidden -->
 		<?php if ($readonly): ?>
 			<td>
 		<?php else: ?>
 			<td class="<?= ZBX_STYLE_TD_DRAG_ICON ?>">
 				<div class="<?= ZBX_STYLE_DRAG_ICON ?>"></div>
-				<span class="ui-icon ui-icon-arrowthick-2-n-s move"></span>
 		<?php endif ?>
 			<input type="hidden" id="items_#{number}_gitemid" name="items[#{number}][gitemid]" value="#{gitemid}">
 			<input type="hidden" id="items_#{number}_itemid" name="items[#{number}][itemid]" value="#{itemid}">
@@ -129,7 +122,7 @@
 
 		<!-- row number -->
 		<td>
-			<span id="items_#{number}_number" class="items_number">#{number_nr}:</span>
+			<span class="<?= ZBX_STYLE_LIST_NUMBERED_ITEM ?>">:</span>
 		</td>
 
 		<!-- name -->
@@ -181,14 +174,13 @@
 </script>
 
 <script type="text/x-jquery-tmpl" id="tmpl-item-row-<?= GRAPH_TYPE_PIE ?>">
-	<tr id="items_#{number}" class="sortable">
+	<tr id="items_#{number}" class="graph-item">
 		<!-- icon + hidden -->
 		<?php if ($readonly): ?>
 			<td>
 		<?php else: ?>
 			<td class="<?= ZBX_STYLE_TD_DRAG_ICON ?>">
 				<div class="<?= ZBX_STYLE_DRAG_ICON ?>"></div>
-				<span class="ui-icon ui-icon-arrowthick-2-n-s move"></span>
 		<?php endif ?>
 			<input type="hidden" id="items_#{number}_gitemid" name="items[#{number}][gitemid]" value="#{gitemid}">
 			<input type="hidden" id="items_#{number}_itemid" name="items[#{number}][itemid]" value="#{itemid}">
@@ -202,7 +194,7 @@
 
 		<!-- row number -->
 		<td>
-			<span id="items_#{number}_number" class="items_number">#{number_nr}:</span>
+			<span class="<?= ZBX_STYLE_LIST_NUMBERED_ITEM ?>">:</span>
 		</td>
 
 		<!-- name -->
@@ -255,14 +247,13 @@
 </script>
 
 <script type="text/x-jquery-tmpl" id="tmpl-item-row-<?= GRAPH_TYPE_EXPLODED ?>">
-	<tr id="items_#{number}" class="sortable">
+	<tr id="items_#{number}" class="graph-item">
 		<!-- icon + hidden -->
 		<?php if ($readonly): ?>
 			<td>
 		<?php else: ?>
 			<td class="<?= ZBX_STYLE_TD_DRAG_ICON ?>">
 				<div class="<?= ZBX_STYLE_DRAG_ICON ?>"></div>
-				<span class="ui-icon ui-icon-arrowthick-2-n-s move"></span>
 		<?php endif ?>
 			<input type="hidden" id="items_#{number}_gitemid" name="items[#{number}][gitemid]" value="#{gitemid}">
 			<input type="hidden" id="items_#{number}_itemid" name="items[#{number}][itemid]" value="#{itemid}">
@@ -276,7 +267,7 @@
 
 		<!-- row number -->
 		<td>
-			<span id="items_#{number}_number" class="items_number">#{number_nr}:</span>
+			<span class="<?= ZBX_STYLE_LIST_NUMBERED_ITEM ?>">:</span>
 		</td>
 
 		<!-- name -->
@@ -332,15 +323,18 @@
 	const view = {
 		form_name: null,
 		graphs: null,
+		context: null,
+		parent_discoveryid: null,
 
-		init({form_name, theme_colors, graphs, items}) {
+		init({form_name, theme_colors, graphs, items, context, parent_discoveryid}) {
 			this.form_name = form_name;
 			colorPalette.setThemeColors(theme_colors);
 			this.graphs = graphs;
+			this.context = context;
+			this.is_discovery = parent_discoveryid !== null;
 
 			items.forEach((item, i) => {
 				item.number = i;
-				item.number_nr = i + 1;
 				item.name = item.host + '<?= NAME_DELIMITER ?>' + item.name;
 
 				this.loadItem(item);
@@ -363,6 +357,7 @@
 					src.setArgument('height', $('#height').val());
 					src.setArgument('graphtype', $('#graphtype').val());
 					src.setArgument('legend', $('#show_legend').is(':checked') ? 1 : 0);
+					src.setArgument('resolve_macros', this.context === 'template' ? 0 : 1);
 
 					if (this.graphs.graphtype == <?= GRAPH_TYPE_PIE ?>
 							|| this.graphs.graphtype == <?= GRAPH_TYPE_EXPLODED ?>) {
@@ -399,7 +394,7 @@
 						src.setArgument('showtriggers', $('#show_triggers').is(':checked') ? 1 : 0);
 					}
 
-					$('#itemsTable tr.sortable').each((i, node) => {
+					$('#itemsTable tbody tr.graph-item').each((i, node) => {
 						const short_fmt = [];
 
 						$(node).find('*[name]').each((_, input) => {
@@ -432,7 +427,7 @@
 			});
 
 			if (this.graphs.readonly) {
-				const size = $('#itemsTable tr.sortable').length;
+				const size = $('#itemsTable tbody tr.graph-item').length;
 
 				for (let i = 0; i < size; i++) {
 					$('#items_' + i + '_color')
@@ -490,7 +485,14 @@
 				$('form[name="' + view.form_name + '"]').submit();
 			});
 
-			!this.graphs.readonly && this.initSortable();
+			new CSortable(document.querySelector('#itemsTable tbody'), {
+				selector_handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
+				freeze_end: 1,
+				enable_sorting: !this.graphs.readonly
+			})
+				.on(CSortable.EVENT_SORT, this.recalculateSortOrder);
+
+			!this.graphs.readonly && this.rewriteNameLinks();
 		},
 
 		loadItem(item) {
@@ -523,10 +525,9 @@
 					}
 				}
 
-				const number = $('#itemsTable tr.sortable').length;
+				const number = $('#itemsTable tbody tr.graph-item').length;
 				const item = {
 					number: number,
-					number_nr: number + 1,
 					gitemid: null,
 					itemid: list.values[i].itemid,
 					calc_fnc: null,
@@ -544,10 +545,7 @@
 				$(`#items_${number}_color`).colorpicker();
 			}
 
-			if (!this.graphs.readonly) {
-				this.activateSortable();
-				this.rewriteNameLinks();
-			}
+			!this.graphs.readonly && this.rewriteNameLinks();
 		},
 
 		getOnlyHostParam() {
@@ -557,7 +555,7 @@
 		},
 
 		rewriteNameLinks() {
-			const size = $('#itemsTable tr.sortable').length;
+			const size = $('#itemsTable tbody tr.graph-item').length;
 
 			for (let i = 0; i < size; i++) {
 				const parameters = {
@@ -602,26 +600,25 @@
 			$('#items_' + number).remove();
 
 			this.recalculateSortOrder();
-			!this.graphs.readonly && this.activateSortable();
 		},
 
 		recalculateSortOrder() {
 			let i = 0;
 
 			// Rewrite IDs, set "tmp" prefix.
-			$('#itemsTable tr.sortable').find('*[id]').each(function() {
+			$('#itemsTable tbody tr.graph-item').find('*[id]').each(function() {
 				const $obj = $(this);
 
 				$obj.attr('id', 'tmp' + $obj.attr('id'));
 			});
 
-			$('#itemsTable tr.sortable').each(function() {
+			$('#itemsTable tbody tr.graph-item').each(function() {
 				const $obj = $(this);
 
 				$obj.attr('id', 'tmp' + $obj.attr('id'));
 			});
 
-			for (const [index, row] of document.querySelectorAll('#itemsTable tr.sortable').entries()) {
+			for (const [index, row] of document.querySelectorAll('#itemsTable tbody tr.graph-item').entries()) {
 				row.id = row.id.substring(3).replace(/\d+/, `${index}`);
 
 				row.querySelectorAll('[id]').forEach(element => {
@@ -637,10 +634,7 @@
 				});
 			}
 
-			$('#itemsTable tr.sortable').each(function() {
-				// Set row number.
-				$('.items_number', this).text((i + 1) + ':');
-
+			$('#itemsTable tbody tr.graph-item').each(function() {
 				// Set remove number.
 				$('#items_' + i + '_remove').data('remove', i);
 
@@ -650,38 +644,6 @@
 			!view.graphs.readonly && view.rewriteNameLinks();
 		},
 
-		initSortable() {
-			$('#itemsTable').sortable({
-				disabled: ($('#itemsTable tr.sortable').length < 2),
-				items: 'tbody tr.sortable',
-				axis: 'y',
-				containment: 'parent',
-				cursor: 'grabbing',
-				handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
-				tolerance: 'pointer',
-				opacity: 0.6,
-				update: this.recalculateSortOrder,
-				helper: (e, ui) => {
-					for (const td of ui.find('>td')) {
-						const $td = $(td);
-						$td.attr('width', $td.width())
-					}
-
-					return ui;
-				},
-				stop: (e, ui) => {
-					ui.item.find('>td').removeAttr('width');
-				},
-				start: (e, ui) => {
-					$(ui.placeholder).height($(ui.helper).height());
-				}
-			});
-		},
-
-		activateSortable() {
-			$('#itemsTable').sortable({disabled: ($('#itemsTable tr.sortable').length < 2)});
-		},
-
 		editHost(e, hostid) {
 			e.preventDefault();
 			const host_data = {hostid};
@@ -697,14 +659,33 @@
 				prevent_navigation: true
 			});
 
-			overlay.$dialogue[0].addEventListener('dialogue.create', this.events.hostSuccess, {once: true});
-			overlay.$dialogue[0].addEventListener('dialogue.update', this.events.hostSuccess, {once: true});
-			overlay.$dialogue[0].addEventListener('dialogue.delete', this.events.hostDelete, {once: true});
-			overlay.$dialogue[0].addEventListener('overlay.close', () => {
+			overlay.$dialogue[0].addEventListener('dialogue.submit',
+				this.events.elementSuccess.bind(this, this.context, this.is_discovery), {once: true}
+			);
+			overlay.$dialogue[0].addEventListener('dialogue.close', () => {
 				history.replaceState({}, '', original_url);
 			}, {once: true});
 		},
 
+		editTemplate(e, templateid) {
+			e.preventDefault();
+			const template_data = {templateid};
+
+			this.openTemplatePopup(template_data);
+		},
+
+		openTemplatePopup(template_data) {
+			const overlay =  PopUp('template.edit', template_data, {
+				dialogueid: 'templates-form',
+				dialogue_class: 'modal-popup-large',
+				prevent_navigation: true
+			});
+
+			overlay.$dialogue[0].addEventListener('dialogue.submit',
+				this.events.elementSuccess.bind(this, this.context, this.is_discovery), {once: true}
+			);
+		},
+
 		refresh() {
 			const url = new Curl('');
 			const form = document.getElementsByName(this.form_name)[0];
@@ -714,8 +695,9 @@
 		},
 
 		events: {
-			hostSuccess(e) {
+			elementSuccess(context, discovery, e) {
 				const data = e.detail;
+				let curl = null;
 
 				if ('success' in data) {
 					postMessageOk(data.success.title);
@@ -723,26 +705,19 @@
 					if ('messages' in data.success) {
 						postMessageDetails('success', data.success.messages);
 					}
-				}
 
-				view.refresh();
-			},
-
-			hostDelete(e) {
-				const data = e.detail;
-
-				if ('success' in data) {
-					postMessageOk(data.success.title);
-
-					if ('messages' in data.success) {
-						postMessageDetails('success', data.success.messages);
+					if ('action' in data.success && data.success.action === 'delete') {
+						curl = discovery ? new Curl('host_discovery.php') : new Curl('graphs.php');
+						curl.setArgument('context', context);
 					}
 				}
 
-				const curl = new Curl('zabbix.php');
-				curl.setArgument('action', 'host.list');
-
-				location.href = curl.getUrl();
+				if (curl === null) {
+					view.refresh();
+				}
+				else {
+					location.href = curl.getUrl();
+				}
 			}
 		}
 	};
