@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -148,6 +143,7 @@ if ($data['action'] == HISTORY_LATEST || $data['action'] == HISTORY_VALUES) {
 								'srcfld1' => 'itemid',
 								'dstfld1' => 'itemids_',
 								'real_hosts' => true,
+								'resolve_macros' => true,
 								'value_types' => [ITEM_VALUE_TYPE_LOG, ITEM_VALUE_TYPE_TEXT]
 							]
 						]
@@ -226,7 +222,8 @@ if ($data['plaintext']) {
 		foreach ($screen as $text) {
 			$pre->addItem([$text, BR()]);
 		}
-		$html_page->addItem($pre);
+
+		$html_page->addItem($pre->addClass(ZBX_STYLE_NOWRAP));
 	}
 }
 else {
@@ -237,10 +234,11 @@ else {
 
 	if ($data['itemids'] && $data['action'] !== HISTORY_LATEST) {
 		$filter_form->addTimeSelector($screen->timeline['from'], $screen->timeline['to'],
-			$web_layout_mode != ZBX_LAYOUT_KIOSKMODE);
+			$web_layout_mode != ZBX_LAYOUT_KIOSKMODE
+		);
 	}
 
-	if ($data['action'] == HISTORY_BATCH_GRAPH) {
+	if (($data['action'] == HISTORY_BATCH_GRAPH || $data['action'] == HISTORY_GRAPH) && count($data['itemids']) > 1) {
 		$filter_form
 			->hideFilterButtons()
 			->addVar('action', $data['action'])
@@ -288,7 +286,6 @@ else {
 					(new CColHeader(_('Local time')))->addClass(ZBX_STYLE_CELL_WIDTH),
 					_('Value')
 				])
-				->setNoDataMessage(_('Specify some filter condition to see the values.'))
 		);
 	}
 }
