@@ -1,20 +1,15 @@
 /*
-** Zabbix
 ** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -23,6 +18,8 @@
 	const MS_ACTION_POPUP = 0;
 	const MS_ACTION_AUTOSUGGEST = 1;
 
+	const FILTER_PRESELECT_ACCEPT_ID = 'id';
+
 	/**
 	 * Multi select helper.
 	 *
@@ -35,7 +32,7 @@
 		options = $.extend({objectOptions: {}}, options);
 
 		var curl = new Curl('jsrpc.php');
-		curl.setArgument('type', 11); // PAGE_TYPE_TEXT_RETURN_JSON
+		curl.setArgument('type', PAGE_TYPE_TEXT_RETURN_JSON);
 		curl.setArgument('method', 'multiselect.get');
 		curl.setArgument('object_name', options.object_name);
 
@@ -46,7 +43,9 @@
 		options.url = curl.getUrl();
 
 		return this.each(function() {
-			$(this).empty().multiSelect(options);
+			$(this).empty();
+			this.dataset.params = JSON.stringify(options);
+			$(this).multiSelect();
 		});
 	};
 
@@ -90,8 +89,8 @@
 		/**
 		 * Insert outside data
 		 *
-		 * @param {array} items           Multiselect value object.
-		 * @param {bool}  trigger_change  (optional) Either to trigger element on-change event once data added. True by default.
+		 * @param {array}   items           Multiselect value object.
+		 * @param {boolean} trigger_change  (optional) Either to trigger element on-change event once data added. True by default.
 		 *
 		 * @return jQuery
 		 */
@@ -110,6 +109,8 @@
 					}
 				}
 
+				$obj.trigger('before-add', ms);
+
 				for (var i = 0, l = items.length; i < l; i++) {
 					addSelected($obj, items[i]);
 				}
@@ -118,6 +119,18 @@
 			});
 		},
 
+		getSearch: function() {
+			let search = '';
+
+			this.each(function() {
+				const ms = $(this).data('multiSelect');
+
+				search = ms.values.search.toLowerCase().replace(/[*]+/g, '')
+			});
+
+			return search;
+		},
+
 		/**
 		 * Enable multi select UI control.
 		 *
@@ -128,23 +141,16 @@
 				var $obj = $(this),
 					ms = $obj.data('multiSelect');
 
-				if (ms.options.disabled === true || ms.options.readonly === true) {
-					if (ms.options.disabled === true) {
-						$obj.removeAttr('aria-disabled');
-						$('.multiselect-list', $obj).removeClass('disabled');
-
-						ms.options.disabled = false;
-					}
-					else {
-						$obj.removeAttr('aria-readonly');
-						$('.multiselect-list', $obj).removeAttr('aria-readonly');
-
-						ms.options.readonly = false;
-					}
-
-					$('.multiselect-button > button', $obj.parent()).prop('disabled', false);
+				if (ms.options.disabled === true) {
+					$obj.removeAttr('aria-disabled');
+					$('.multiselect-list', $obj).removeClass('disabled');
+					$('.multiselect-button', $obj.parent()).prop('disabled', false);
+					$('.multiselect-optional-select-button', $obj.parent()).prop('disabled', false);
+					$('input', $obj).prop('disabled', false);
 					$obj.append(makeMultiSelectInput($obj));
 
+					ms.options.disabled = false;
+
 					cleanSearch($obj);
 				}
 			});
@@ -163,8 +169,10 @@
 				if (ms.options.disabled === false) {
 					$obj.attr('aria-disabled', true);
 					$('.multiselect-list', $obj).addClass('disabled');
-					$('.multiselect-button > button', $obj.parent()).prop('disabled', true);
+					$('.multiselect-button', $obj.parent()).prop('disabled', true);
+					$('.multiselect-optional-select-button', $obj.parent()).prop('disabled', true);
 					$('input[type="text"]', $obj).remove();
+					$('input', $obj).prop('disabled', true);
 
 					ms.options.disabled = true;
 
@@ -173,6 +181,26 @@
 			});
 		},
 
+		/**
+		 * Remove select object value.
+		 *
+		 * @param {string} id
+		 *
+		 * @return jQuery
+		 */
+		removeSelected: function(id) {
+			return this.each(function() {
+				var $obj = $(this),
+					ms = $obj.data('multiSelect');
+
+				removeSelected($obj, id);
+
+				cleanSearch($obj);
+
+				$obj.trigger('change', ms);
+			});
+		},
+
 		/**
 		 * Clean multi select object values.
 		 *
@@ -211,6 +239,8 @@
 					}
 				}
 
+				cleanSearch($obj);
+
 				if (addNew_modified) {
 					/*
 					 * When modifying the "addNew" option, few things must be done:
@@ -220,8 +250,6 @@
 					 *   3. The "change" trigger must fire.
 					 */
 
-					cleanSearch($obj);
-
 					$('input[name*="[new]"]', $obj)
 						.prop('disabled', !ms.options['addNew'])
 						.each(function() {
@@ -266,7 +294,9 @@
 				var $obj = $(this);
 
 				if ($obj.data('multiSelect') !== undefined) {
-					ret = $obj.data('multiSelect').select_button[0];
+					if ($obj.data('multiSelect').select_button !== null) {
+						ret = $obj.data('multiSelect').select_button[0];
+					}
 
 					return false;
 				}
@@ -278,78 +308,124 @@
 		/**
 		 * @param array entries  IDs to mark disabled.
 		 */
-		setDisabledEntries: function (entries) {
+		setDisabledEntries: function(entries) {
 			this.each(function() {
 				const $obj = $(this);
-				const ms_parameters = $obj.data('multiSelect');
+				const ms = $obj.data('multiSelect');
 
-				if (typeof ms_parameters === 'undefined') {
-					return;
+				if (ms?.options.popup.parameters !== undefined) {
+					ms.options.popup.parameters.disableids = entries;
 				}
+			});
+		},
 
-				const link = new Curl(ms_parameters.options.url);
-				link.setArgument('disabledids', entries);
+		addOptionalSelect: function(label, callback) {
+			this.each(function() {
+				const $obj = $(this);
 
-				ms_parameters.options.url = link.getUrl();
-				ms_parameters.options.popup.parameters.disableids = entries;
+				if ($obj.data('multiSelect') !== undefined) {
+					addOptionalSelect($obj, label, callback);
 
-				$obj.data('multiSelect', ms_parameters);
+					return false;
+				}
+			});
+		},
+
+		openSelectPopup: function(event_target) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				if (ms !== undefined && ms.options.popup.parameters !== undefined) {
+					openSelectPopup($obj, event_target);
+
+					return false;
+				}
+			});
+		},
+
+		setCustomSuggestList: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.custom_suggest_list = callback;
+			});
+		},
+
+		setSuggestListModifier: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.suggest_list_modifier = callback;
+			});
+		},
+
+		customSuggestSelectHandler: function(callback) {
+			this.each(function() {
+				const $obj = $(this);
+				const ms = $obj.data('multiSelect');
+
+				ms.options.custom_suggest_select_handler = callback;
 			});
 		}
 	};
 
 	/**
-	 * Create multi select input element.
+	 * Initialize and interact with multi select input element.
 	 *
-	 * @param string options['url']					backend url
-	 * @param string options['name']				input element name
-	 * @param string options['multiselect_id']      multiselect wrapper id (optional)
-	 * @param object options['labels']				translated labels (optional)
-	 * @param object options['data']				preload data {id, name, prefix} (optional)
-	 * @param string options['data'][id]
-	 * @param string options['data'][name]
-	 * @param string options['data'][prefix]		(optional)
-	 * @param bool   options['data'][inaccessible]	(optional)
-	 * @param bool   options['data'][disabled]		(optional)
-	 * @param string options['placeholder']			set custom placeholder (optional)
-	 * @param array  options['excludeids']			the list of excluded ids (optional)
-	 * @param string options['defaultValue']		default value for input element (optional)
-	 * @param bool   options['disabled']			turn on/off disabled state (optional)
-	 * @param bool   options['readonly']		    turn on/off readonly state (optional)
-	 * @param bool   options['addNew']				allow user to create new names (optional)
-	 * @param int    options['selectedLimit']		how many items can be selected (optional)
-	 * @param int    options['limit']				how many available items can be received from backend (optional)
-	 * @param object options['popup']				popup data {parameters, width, height} (optional)
-	 * @param string options['popup']['parameters']
-	 * @param string options['popup']['filter_preselect']
-	 * @param string options['popup']['filter_preselect']['id']
-	 * @param string options['popup']['filter_preselect']['submit_as']
-	 * @param object options['popup']['filter_preselect']['submit_parameters']
-	 * @param bool   options['popup']['filter_preselect']['multiple']
-	 * @param int    options['popup']['width']
-	 * @param int    options['popup']['height']
-	 * @param object options['autosuggest']         autosuggest options (optional)
-	 * @param object options['autosuggest']['filter_preselect']
-	 * @param string options['autosuggest']['filter_preselect']['id']
-	 * @param string options['autosuggest']['filter_preselect']['submit_as']
-	 * @param object options['autosuggest']['filter_preselect']['submit_parameters']
-	 * @param bool   options['autosuggest']['filter_preselect']['multiple']
-	 * @param string options['styles']				additional style for multiselect wrapper HTML element (optional)
-	 * @param string options['styles']['property']
-	 * @param string options['styles']['value']
+	 * Function can either accept a method from supported methods or expects the multiselect element to contain
+	 * a 'data-params' attribute with the following possible properties:
+	 *     string url                   backend url
+	 *     string name                  input element name
+	 *     string multiselect_id        multiselect wrapper id (optional)
+	 *     object labels                translated labels (optional)
+	 *     array  data                  preload data {id, name, prefix} (optional)
+	 *     string data[][id]
+	 *     string data[][name]
+	 *     string data[][prefix]        (optional)
+	 *     bool   data[][inaccessible]  (optional)
+	 *     bool   data[][disabled]      (optional)
+	 *     string placeholder           set custom placeholder (optional)
+	 *     string defaultValue          default value for input element (optional)
+	 *     bool   disabled              turn on/off disabled state (optional)
+	 *     bool   readonly              turn on/off readonly state (optional)
+	 *     bool   hidden                hide element (optional)
+	 *     bool   addNew                allow user to create new names (optional)
+	 *     int    selectedLimit         how many items can be selected (optional)
+	 *     int    limit                 how many available items can be received from backend (optional)
+	 *     object popup                 popup data {parameters, width, height} (optional)
+	 *     string popup[parameters]
+	 *     string popup[filter_preselect]
+	 *     string popup[filter_preselect][id]
+	 *     string popup[filter_preselect][submit_as]
+	 *     object popup[filter_preselect][submit_parameters]
+	 *     bool   popup[filter_preselect][multiple]
+	 *     int    popup[width]
+	 *     int    popup[height]
+	 *     object autosuggest           autosuggest options (optional)
+	 *     object autosuggest[filter_preselect]
+	 *     string autosuggest[filter_preselect][id]
+	 *     string autosuggest[filter_preselect][submit_as]
+	 *     object autosuggest[filter_preselect][submit_parameters]
+	 *     bool   autosuggest[filter_preselect][multiple]
+	 *     string styles                additional style for multiselect wrapper HTML element (optional)
+	 *     string styles[property]
+	 *     string styles[value]
 	 *
 	 * @return object
 	 */
-	$.fn.multiSelect = function(options) {
-		// Call a public method.
-		if (methods[options]) {
-			return methods[options].apply(this, Array.prototype.slice.call(arguments, 1));
+	$.fn.multiSelect = function(method) {
+		if (method !== undefined) {
+			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
 		}
 
 		var defaults = {
 			url: '',
 			name: '',
 			multiselect_id: '',
+			object_labels: {object: '', objects: ''},
 			labels: {
 				'No matches found': t('No matches found'),
 				'More matches found...': t('More matches found...'),
@@ -359,9 +435,12 @@
 			},
 			placeholder: t('type here to search'),
 			data: [],
-			excludeids: [],
 			addNew: false,
 			defaultValue: null,
+			custom_select: false,
+			custom_suggest_list: null,
+			suggest_list_modifier: null,
+			custom_suggest_select_handler: null,
 			disabled: false,
 			readonly: false,
 			selectedLimit: 0,
@@ -370,8 +449,6 @@
 			styles: {}
 		};
 
-		options = $.extend({}, defaults, options);
-
 		return this.each(function() {
 			var $obj = $(this);
 
@@ -379,27 +456,30 @@
 				return;
 			}
 
+			const options = $.extend({}, defaults, JSON.parse(this.dataset.params));
+
 			options.required_str = $obj.attr('aria-required') === undefined ? 'false' : $obj.attr('aria-required');
 			$obj.removeAttr('aria-required');
 
 			var ms = {
-					options: options,
-					values: {
-						search: '',
-						searches: {},
-						searching: {},
-						selected: {},
-						available: {},
-						available_div: $('<div>', {'class': 'multiselect-available'}),
+				options: options,
+				values: {
+					search: '',
+					searches: {},
+					searching: {},
+					selected: {},
+					available: new Map(),
+					available_div: $('<div>', {'class': 'multiselect-available'}),
 
-						/*
-						 * Indicates a false click on an available list, but not on some actual item.
-						 * In such case the "focusout" event (IE) of the search input should not be processed.
-						 */
-						available_false_click: false
-					},
-					select_button: null
-				};
+					/*
+					 * Indicates a false click on an available list, but not on some actual item.
+					 * In such case the "focusout" event (IE) of the search input should not be processed.
+					 */
+					available_false_click: false
+				},
+				select_button: null,
+				optional_select_menu: []
+			};
 
 			ms.values.available_div.on('mousedown', 'li', function() {
 				/*
@@ -418,13 +498,13 @@
 			}));
 
 			var $selected_div = $('<div>', {'class': 'selected'}).on('click', function() {
-					/*
-					 * Focus without options because here it don't matter.
-					 * Click used instead focus because in patternselect listen only click.
-					 */
-					$('input[type="text"]', $obj).click().focus();
-				}),
-				$selected_ul = $('<ul>', {'class': 'multiselect-list'});
+				/*
+				 * Focus without options because here it don't matter.
+				 * Click used instead focus because in patternselect listen only click.
+				 */
+				$('input[type="text"]', $obj).click().focus();
+			}),
+			$selected_ul = $('<ul>', {'class': 'multiselect-list'});
 
 			$obj.append($selected_div.append($selected_ul));
 
@@ -470,37 +550,21 @@
 			if (ms.options.custom_select || ms.options.popup.parameters !== undefined) {
 				ms.select_button = $('<button>', {
 					type: 'button',
-					'class': 'btn-grey',
+					'class': `${ZBX_STYLE_BTN_GREY} multiselect-button`,
 					text: ms.options.labels['Select']
 				});
+			}
+
+			if (ms.select_button !== null) {
+				if (ms.options.popup.parameters !== undefined) {
+					ms.select_button.on('click', (e) => openSelectPopup($obj, e.target));
+				}
 
 				if (ms.options.disabled || ms.options.readonly) {
 					ms.select_button.prop('disabled', true);
 				}
 
-				if (ms.options.popup.parameters !== undefined) {
-					ms.select_button.on('click', function(event) {
-						var parameters = ms.options.popup.parameters;
-
-						if (ms.options.popup.filter_preselect) {
-							parameters = jQuery.extend(parameters, getFilterPreselect($obj, MS_ACTION_POPUP));
-						}
-
-						if (typeof parameters['disable_selected'] !== 'undefined' && parameters['disable_selected']) {
-							parameters['disableids'] = Object.keys(ms.values.selected);
-						}
-
-						// Click used instead focus because in patternselect only click is listened for.
-						$('input[type="text"]', $obj).click();
-
-						return PopUp('popup.generic', parameters, {
-							dialogue_class: 'modal-popup-generic',
-							trigger_element: event.target
-						});
-					});
-				}
-
-				$obj.after($('<div>', {'class': 'multiselect-button'}).append(ms.select_button));
+				addSelectButton($obj, ms.select_button);
 			}
 		});
 	};
@@ -524,22 +588,23 @@
 		const filter_preselect = ms.options[options_key].filter_preselect;
 		const data = $('#' + filter_preselect.id).multiSelect('getData');
 
-		if (data.length === 0) {
+		const accept = filter_preselect.accept ?? null;
+
+		const ret_data = [];
+
+		for (const item of data) {
+			if (accept === null || (accept === FILTER_PRESELECT_ACCEPT_ID && /^[0-9]+$/g.test(item.id))) {
+				ret_data.push(item.id);
+			}
+		}
+
+		if (ret_data.length === 0) {
 			return {};
 		}
 
-		let ret = {};
-
-		if ('multiple' in filter_preselect && filter_preselect.multiple) {
-			ret[filter_preselect.submit_as] = [];
-
-			for (const item of data) {
-				ret[filter_preselect.submit_as].push(item.id);
-			}
-		}
-		else {
-			ret[filter_preselect.submit_as] = data[0].id;
-		}
+		let ret = {
+			[filter_preselect.submit_as]: filter_preselect.multiple ? ret_data : ret_data[0]
+		};
 
 		if ('submit_parameters' in filter_preselect) {
 			ret = {...ret, ...filter_preselect.submit_parameters};
@@ -597,7 +662,14 @@
 						 * 3. Schedule result set retrieval for the given term otherwise.
 						 */
 
-						if (cache_key in ms.values.searches) {
+						if (ms.options.custom_suggest_list !== null) {
+							ms.values.search = search;
+							ms.values.cache_key = cache_key;
+							ms.values.searches[cache_key] = ms.options.custom_suggest_list();
+							loadAvailable($obj);
+							showAvailable($obj);
+						}
+						else if (cache_key in ms.values.searches) {
 							ms.values.search = search;
 							ms.values.cache_key = cache_key;
 							loadAvailable($obj);
@@ -614,7 +686,9 @@
 									cache: false,
 									data: jQuery.extend({
 										search: search,
-										limit: getLimit($obj),
+										limit: ms.options.limit !== 0
+											? ms.options.limit + getSkipSearchIds($obj).length + 1
+											: undefined
 									}, preselect_values)
 								})
 									.then(function(response) {
@@ -700,13 +774,15 @@
 
 						case KEY_ARROW_UP:
 						case KEY_ARROW_DOWN:
-							var $collection = $('li', ms.values.available_div.filter(':visible')),
-								$selected = $collection.filter('.suggest-hover').removeClass('suggest-hover');
+							var $collection = $('li[data-id]', ms.values.available_div.filter(':visible')),
+								$selected = $collection.filter('.suggest-hover');
 
 							if ($selected.length) {
+								$selected.removeClass('suggest-hover');
+
 								$selected = (e.which == KEY_ARROW_UP)
-									? ($selected.is(':first-child') ? $collection.last() : $selected.prev())
-									: ($selected.is(':last-child') ? $collection.first() : $selected.next());
+									? ($selected.is($collection.first()) ? $collection.last() : $selected.prevAll('[data-id]').first())
+									: ($selected.is($collection.last()) ? $collection.first() : $selected.nextAll('[data-id]').first());
 
 								$selected.addClass('suggest-hover');
 								$aria_live.text($selected.data('label'));
@@ -729,15 +805,21 @@
 										var aria_text = sprintf(t('Removed, %1$s'), $selected.data('label'));
 
 										$selected = (e.which == KEY_BACKSPACE)
-											? ($selected.is(':first-child') ? $selected.next() : $selected.prev())
-											: ($selected.is(':last-child') ? $selected.prev() : $selected.next());
+											? ($selected.is(':first-child')
+												? $selected.next('[data-id]')
+												: $selected.prev('[data-id]')
+											)
+											: ($selected.is(':last-child')
+												? $selected.prev('[data-id]')
+												: $selected.next('[data-id]')
+											);
 
 										removeSelected($obj, id);
 
 										$obj.trigger('change', ms);
 
 										if ($selected.length) {
-											var $collection = $('.selected li', $obj);
+											var $collection = $('.selected li[data-id]', $obj);
 											$selected.addClass('selected');
 
 											aria_text += ', ' + sprintf(
@@ -770,10 +852,10 @@
 							break;
 					}
 				})
-				.on('focusin', function($event) {
+				.on('focusin', function() {
 					$obj.addClass('active');
 				})
-				.on('focusout', function($event) {
+				.on('focusout', function() {
 					if (ms.values.available_false_click) {
 						ms.values.available_false_click = false;
 						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
@@ -809,7 +891,12 @@
 	function select($obj, id) {
 		var ms = $obj.data('multiSelect');
 
-		addSelected($obj, ms.values.available[id]);
+		if (ms.options.custom_suggest_select_handler !== null) {
+			ms.options.custom_suggest_select_handler(ms.values.available.get(id.toString()));
+		}
+		else {
+			addSelected($obj, ms.values.available.get(id.toString()));
+		}
 
 		if (isSearchFieldVisible($obj)) {
 			$('input[type="text"]', $obj)[0].focus({preventScroll:true});
@@ -818,6 +905,59 @@
 		$obj.trigger('change', ms);
 	}
 
+	function addSelectButton($obj, $button) {
+		let $container = $obj.siblings('.btn-split');
+
+		if (!$container.length) {
+			$container = $('<ul>', {'class': 'btn-split'});
+			$obj.after($container);
+		}
+
+		$container.append($('<li>').append($button));
+	}
+
+	function addOptionalSelect($obj, label, callback) {
+		const ms = $obj.data('multiSelect');
+
+		if (!ms.optional_select_menu.length) {
+			addSelectButton($obj, $('<button>', {
+				type: 'button',
+				class: `${ZBX_STYLE_BTN_GREY} ${ZBX_ICON_CHEVRON_DOWN_SMALL} multiselect-optional-select-button`
+			}).on('click', function(event) {
+				jQuery(event.target).menuPopup(
+					[{items: ms.optional_select_menu}],
+					new jQuery.Event(event), {
+						position: {of: event.target, my: 'left top', at: 'left bottom', within: '.wrapper'}
+					}
+				);
+			}));
+		}
+
+		ms.optional_select_menu.push({label, clickCallback: callback});
+	}
+
+	function openSelectPopup($obj, open_trigger_element) {
+		const ms = $obj.data('multiSelect');
+
+		let parameters = ms.options.popup.parameters;
+
+		if (ms.options.popup.filter_preselect) {
+			parameters = jQuery.extend(parameters, getFilterPreselect($obj, MS_ACTION_POPUP));
+		}
+
+		if (parameters['disable_selected'] !== undefined && parameters['disable_selected']) {
+			parameters['disableids'] = Object.keys(ms.values.selected);
+		}
+
+		// Click used instead focus because in pattern select only click is listened for.
+		$('input[type="text"]', $obj).click();
+
+		PopUp('popup.generic', parameters, {
+			dialogue_class: 'modal-popup-generic',
+			trigger_element: open_trigger_element
+		});
+	}
+
 	function addSelected($obj, item) {
 		var ms = $obj.data('multiSelect');
 
@@ -852,7 +992,7 @@
 							title: item.name
 						}))
 						.append($('<span>')
-							.addClass('subfilter-disable-btn')
+							.addClass([ZBX_STYLE_BTN_ICON, ZBX_ICON_REMOVE_SMALLER])
 							.on('click', function() {
 								if (!ms.options.disabled && !item_disabled && !ms.options.readonly) {
 									removeSelected($obj, item.id);
@@ -891,20 +1031,22 @@
 	function removeSelected($obj, id) {
 		var ms = $obj.data('multiSelect');
 
-		$('.selected li[data-id]', $obj).each(function(){
+		$obj.trigger('before-remove', ms);
+
+		$('.multiselect-list [data-id]', $obj).each(function() {
 			if ($(this).data('id') == id) {
 				$(this).remove();
 			}
 		});
-		$('input', $obj).each(function() {
-			if ($(this).val() !== '' && $(this).val() == id) {
+		$('input[type="hidden"]', $obj).each(function() {
+			if ($(this).val() == id) {
 				$(this).remove();
 			}
 		});
 
 		delete ms.values.selected[id];
 
-		if (!$('.selected li', $obj).length) {
+		if (!Object.keys(ms.values.selected).length) {
 			addDefaultValue($obj);
 		}
 
@@ -917,7 +1059,8 @@
 			prefix = item.prefix || '',
 			$li = $('<li>', {
 				'data-id': item.id,
-				'data-label': prefix + item.name
+				'data-label': prefix + item.name,
+				'data-source': item.source
 			})
 				.on('mouseenter', function() {
 					$('li.suggest-hover', ms.values.available_div).removeClass('suggest-hover');
@@ -1012,40 +1155,49 @@
 			}
 		}
 
-		var available_more = false;
+		const skip_search_ids = getSkipSearchIds($obj);
 
-		$.each(data, function(i, item) {
-			if (ms.options.limit == 0 || objectSize(ms.values.available) < ms.options.limit) {
-				if (typeof ms.values.available[item.id] === 'undefined'
-						&& typeof ms.values.selected[item.id] === 'undefined'
-						&& ms.options.excludeids.indexOf(item.id) === -1) {
-					ms.values.available[item.id] = item;
+		let available_more = false;
+
+		if (ms.options.custom_suggest_list !== null) {
+			ms.values.available = data;
+		}
+		else {
+			$.each(data, function(i, item) {
+				if (ms.options.limit == 0 || ms.values.available.size < ms.options.limit) {
+					if (!skip_search_ids.includes(item.id)) {
+						ms.values.available.set(item.id, item);
+					}
 				}
-			}
-			else {
-				available_more = true;
-			}
-		});
+				else {
+					available_more = true;
+				}
+			});
+		}
 
 		if (addNew) {
-			ms.values.available[ms.values.search] = {
+			ms.values.available.set(ms.values.search, {
 				id: ms.values.search,
 				name: ms.values.search + ' (' + ms.options.labels['new'] + ')',
 				isNew: true
-			};
+			});
 		}
 
 		var found = 0,
 			preselected = '';
 
-		if (objectSize(ms.values.available) == 0) {
+		if (ms.options.suggest_list_modifier !== null) {
+			ms.values.available = ms.options.suggest_list_modifier(ms.values.available);
+		}
+
+		if (ms.values.available.size === 0) {
 			var div = $('<div>', {
-					'class': 'multiselect-matches',
-					text: ms.options.labels['No matches found']
-				})
-					.on('click', function() {
-						$('input[type="text"]', $obj)[0].focus({preventScroll:true});
-					});
+				'class': 'multiselect-matches',
+				text: ms.options.labels['No matches found']
+			})
+				.on('click', function() {
+					$('input[type="text"]', $obj)[0].focus({preventScroll:true});
+				});
 
 			ms.values.available_div.append(div);
 		}
@@ -1055,13 +1207,22 @@
 				'aria-hidden': true
 			}));
 
-			$.each(ms.values.available, function (i, item) {
-				if (found == 0) {
-					preselected = (item.prefix || '') + item.name;
+			for (const item of ms.values.available.values()) {
+				if ('group_label' in item) {
+					$('ul', ms.values.available_div)
+						.addClass('multiselect-suggest-grouped')
+						.append(
+							$('<li>', {class: 'suggest-group'}).text(item.group_label)
+						);
 				}
-				addAvailable($obj, item);
-				found++;
-			});
+				else {
+					if (found === 0) {
+						preselected = (item.prefix || '') + item.name;
+					}
+					addAvailable($obj, item);
+					found++;
+				}
+			}
 		}
 
 		if (found > 0) {
@@ -1127,7 +1288,7 @@
 				$(window).height() + $(window).scrollTop() - available_top - obj_padding_y - 10
 			));
 
-		if (objectSize(ms.values.available) > 0) {
+		if (ms.values.available.size > 0) {
 			available_width_min = Math.max(available_width, 300);
 
 			// Prevent less than 15% width difference for the available list and the input field.
@@ -1151,14 +1312,14 @@
 
 		$available.scrollTop(0);
 
-		if (objectSize(ms.values.available) != 0) {
+		if (ms.values.available.size !== 0) {
 			// Remove selected item selected state.
 			$('.selected li.selected', $obj).removeClass('selected');
 
 			// Pre-select first available item.
-			if ($('li', $available).length > 0) {
+			if ($('li[data-id]', $available).length > 0) {
 				$('li.suggest-hover', $available).removeClass('suggest-hover');
-				$('li:first-child', $available).addClass('suggest-hover');
+				$('li[data-id]', $available).first().addClass('suggest-hover');
 			}
 		}
 
@@ -1187,11 +1348,11 @@
 	}
 
 	function cleanAvailable($obj) {
-		var ms = $obj.data('multiSelect');
+		const ms = $obj.data('multiSelect');
 
 		hideAvailable($obj);
 
-		ms.values.available = {};
+		ms.values.available.clear();
 		ms.values.available_div.empty();
 	}
 
@@ -1205,7 +1366,7 @@
 				selected_top = 0,
 				selected_height = $selected.outerHeight(true);
 
-			if ($('.multiselect-matches', $available)) {
+			if ($('.multiselect-matches', $available).length > 0) {
 				selected_top += $('.multiselect-matches', $available).outerHeight(true);
 			}
 
@@ -1298,11 +1459,13 @@
 		return !$obj.hasClass('search-disabled');
 	}
 
-	function getLimit($obj) {
-		var ms = $obj.data('multiSelect');
+	function getSkipSearchIds($obj) {
+		const ms = $obj.data('multiSelect');
 
-		return (ms.options.limit != 0)
-			? ms.options.limit + objectSize(ms.values.selected) + ms.options.excludeids.length + 1
-			: null;
+		return [...new Set([
+			...Object.keys(ms.values.selected),
+			...(ms.options.popup.parameters?.excludeids || []),
+			...(ms.options.popup.parameters?.disableids || [])
+		])];
 	}
 })(jQuery);
